# coding=utf-8
from __future__ import unicode_literals

from storops.lib.common import Enum, EnumList

__author__ = 'Cedric Zhuang'


class UnityEnum(Enum):
    @property
    def description(self):
        return self.value[1]

    @property
    def index(self):
        return self.value[0]

    @classmethod
    def indices(cls):
        return [i.index for i in cls.get_all()]

    def is_equal(self, value):
        return self.index == value

    def _get_properties(self, dec=0):
        if dec < 0:
            props = {'name': self.name}
        else:
            props = {'name': self.name,
                     'description': self.description,
                     'value': self.index}
        return props

    @classmethod
    def from_int(cls, value):
        for item in cls.get_all():
            if isinstance(item, UnityEnum):
                if item.value == value:
                    ret = item
                    break
        else:
            ret = super(UnityEnum, cls).from_int(value)
        return ret


class UnityEnumList(EnumList):
    @classmethod
    def get_enum_class(cls):
        raise NotImplementedError('enum class of this list is not defined.')


class HealthEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OK = (5, 'OK')
    OK_BUT = (7, 'OK But Minor Warning')
    DEGRADED = (10, 'Degraded')
    MINOR = (15, 'Minor Issue')
    MAJOR = (20, 'Major Issue')
    CRITICAL = (25, 'Critical Issue')
    NON_RECOVERABLE = (30, 'Non Recoverable Issue')


class StorageResourceTypeEnum(UnityEnum):
    FILE_SYSTEM = (1, 'File System')
    CONSISTENCY_GROUP = (2, 'Consistency Group')
    VMWARE_FS = (3, 'VMware FS')
    VMWARE_ISCSI = (4, 'VMware iSCSI')
    LUN = (8, 'LUN')
    VVOL_DATASTORE_FS = (9, 'VVol DataStore FS')
    VVOL_DATASTORE_ISCSI = (10, 'VVol DataStore iSCSI')


class RaidTypeEnum(UnityEnum):
    NONE = (0, 'None')
    RAID5 = (1, 'RAID 5')
    RAID0 = (2, 'RAID 0')
    RAID1 = (3, 'RAID 1')
    RAID3 = (4, 'RAID 3')
    RAID10 = (7, 'RAID 10')
    RAID6 = (10, 'RAID 6')
    MIXED = (12, 'Mixed')
    AUTOMATIC = (48879, 'Automatic')


class RaidTypeEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return RaidTypeEnum


class ReplicationTypeEnum(UnityEnum):
    NONE = (0, 'No Replication')
    LOCAL = (1, 'Local Replication')
    REMOTE = (2, 'Remote Replication')


class NasServerUnixDirectoryServiceEnum(UnityEnum):
    NONE = (0, 'No Directory Service')
    NIS = (2, 'Use NIS Server')
    LDAP = (3, 'Use LDAP Server')
    LOCAL_THEN_NIS = (4, 'Local Then NIS')
    LOCAL_THEN_LDAP = (5, 'Local Then LDAP')


class FilesystemTypeEnum(UnityEnum):
    FILESYSTEM = (1, 'File System')
    VMWARE = (2, 'VMware')


class TieringPolicyEnum(UnityEnum):
    AUTOTIER_HIGH = (0, 'Start Highest and Auto-tier')
    AUTOTIER = (1, 'Auto-tier')
    HIGHEST = (2, 'Highest')
    LOWEST = (3, 'Lowest')
    NO_DATA_MOVEMENT = (4, 'No Data Movement')
    MIXED = (0xffff, 'Different Tier Policies')


class TieringPolicyEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return TieringPolicyEnum


class FSSupportedProtocolEnum(UnityEnum):
    NFS = (0, 'NFS')
    CIFS = (1, 'CIFS')
    MULTI_PROTOCOL = (2, 'Multiprotocol')


class AccessPolicyEnum(UnityEnum):
    NATIVE = (0, 'Native')
    UNIX = (1, 'Unix')
    WINDOWS = (2, 'Windows')


class FSFormatEnum(UnityEnum):
    UFS32 = (0, 'UFS32')
    UFS64 = (2, 'UFS64')


class HostIOSizeEnum(UnityEnum):
    GENERAL_8K = (0x2000, '8K for General Purpose')
    GENERAL_16K = (0x4000, '16K for General Purpose')
    GENERAL_32K = (0x8000, '32K for General Purpose')
    GENERAL_64K = (0x10000, '64K for General Purpose')
    EXCHANGE_2007 = (0x2001, '8K for Exchange 2007')
    EXCHANGE_2010 = (0x8001, '32K for Exchange 2010')
    EXCHANGE_2013 = (0x8002, '32K for Exchange 2013')
    ORACLE = (0x2002, '8K for Oracle DB')
    SQL_SERVER = (0x2003, '8K for MS SQL Server')
    VMWARE_HORIZON = (0x2004, '8K for VMware Horizon VDI')
    SHARE_POINT = (0x8003, '32K for SharePoint')
    SAP = (0x2005, '8K for SAP')


class ResourcePoolFullPolicyEnum(UnityEnum):
    DELETE_ALL_SNAPS = (0, 'Delete All Snaps')
    FAIL_WRITES = (1, 'Fail Writes')


class CIFSTypeEnum(UnityEnum):
    CIFS_SHARE = (1, 'Share on a File System')
    CIFS_SNAPSHOT = (2, 'Share on a Snapshot')


class CifsShareOfflineAvailabilityEnum(UnityEnum):
    MANUAL = (0, 'Manual')
    DOCUMENTS = (1, 'Documents')
    PROGRAMS = (2, 'Programs')
    NONE = (3, 'None')
    INVALID = (255, 'Invalid')


class NFSTypeEnum(UnityEnum):
    NFS_SHARE = (1, 'Share on a File System')
    VMWARE_NFS = (2, 'Share on a VMware Data Store')
    NFS_SNAPSHOT = (3, 'Share on a Snapshot')


class NFSShareRoleEnum(UnityEnum):
    PRODUCTION = (0, "for Production")
    BACKUP = (1, "for Backup")


class NFSShareDefaultAccessEnum(UnityEnum):
    NO_ACCESS = (0, "No Access")
    READ_ONLY = (1, "Read Only")
    READ_WRITE = (2, "Read Write")
    ROOT = (3, "Root")
    RO_ROOT = (4, "Read Only Root")


class NFSShareSecurityEnum(UnityEnum):
    SYS = (0, 'any NFS security types')
    KERBEROS = (1, 'Kerberos')
    KERBEROS_WITH_INTEGRITY = (2, 'Kerberos with integrity')
    KERBEROS_WITH_ENCRYPTION = (3, 'Kerberos with Encryption Security')


class SnapCreatorTypeEnum(UnityEnum):
    NONE = (0, 'Not Specified')
    SCHEDULED = (1, 'Created by Schedule')
    USER_CUSTOM = (2, 'Created by User with a Custom Name')
    USER_DEFAULT = (3, 'Created by User with a Default Name')
    EXTERNAL_VSS = (4, 'Created by VSS')
    EXTERNAL_NDMP = (5, 'Created by NDMP')
    EXTERNAL_RESTORE = (6, 'Created as a Backup before a Restore')
    EXTERNAL_REPLICATION_MANAGER = (8, 'Created by Replication Manger')
    REP_V2 = (9, 'Created by Native Replication')
    INBAND = (11, 'Created by SnapCLI')
    APP_SYNC = (12, 'Created by AppSync')


class SnapStateEnum(UnityEnum):
    READY = (2, 'Ready')
    FAULTED = (3, 'Faulted')
    OFFLINE = (6, 'Offline')
    INVALID = (7, 'Invalid')
    INITIALIZING = (8, 'Initializing')
    DESTROYING = (9, 'Destroying')


class SnapAccessLevelEnum(UnityEnum):
    READ_ONLY = (0, 'Read Only')
    READ_WRITE = (1, 'Read Write')
    READ_ONLY_PARTIAL = (2, 'Read Only Partial')
    READ_WRITE_PARTIAL = (3, 'Read Write Partial')
    MIXED = (4, 'Mixed')


class FilesystemSnapAccessTypeEnum(UnityEnum):
    CHECKPOINT = (1, 'Checkpoint')
    PROTOCOL = (2, 'Protocol')


class RaidStripeWidthEnum(UnityEnum):
    BEST_FIT = (0, '')
    _2 = (2, '2 disk group, usable in RAID10')
    _4 = (4, '4 disk group, usable in RAID10')
    _5 = (5, '5 disk group, usable in RAID5')
    _6 = (6, '6 disk group, usable in RAID6 and RAID10')
    _8 = (8, '8 disk group, usable in RAID6 and RAID10')
    _9 = (9, '9 disk group, usable in RAID5')
    _10 = (10, '10 disk group, usable in RAID6 and RAID10')
    _12 = (12, '12 disk group, usable in RAID6 and RAID10')
    _13 = (13, '13 disk group, usable in RAID5')
    _14 = (14, '14 disk group, usable in RAID6')
    _16 = (16, 'including parity disks, usable in RAID6')


class FastVPStatusEnum(UnityEnum):
    NOT_APPLICABLE = (1, 'Not applicable')
    PAUSED = (2, 'Paused')
    ACTIVE = (3, 'Active')
    NOT_STARTED = (4, 'Not Started')
    COMPLETED = (5, 'Completed')
    STOPPED_BY_USER = (6, 'Stopped by User')
    FAILED = (7, 'Failed')


class FastVPRelocationRateEnum(UnityEnum):
    HIGH = (1, 'High')
    MEDIUM = (2, 'Medium')
    LOW = (3, 'Low')
    NONE = (4, 'None')


class PoolDataRelocationTypeEnum(UnityEnum):
    MANUAL = (1, 'Manual')
    SCHEDULED = (2, 'Scheduled')
    REBALANCE = (3, 'Rebalance')


class UsageHarvestStateEnum(UnityEnum):
    IDLE = (0, 'Idle')
    RUNNING = (1, 'Running')
    COULD_NOT_REACH_LWM = (2, 'Could not Reach LWM')
    PAUSED_COULD_NOT_REACH_HWM = (3, 'Paused Could not Reach LWM')
    FAILED = (4, 'Failed')


class TierTypeEnum(UnityEnum):
    NONE = (0, 'None')
    NVME_EXTREME_PERFORMANCE = (5, 'NVMe_Extreme_Performance')
    EXTREME_PERFORMANCE = (10, 'Extreme Performance')
    PERFORMANCE = (20, 'Performance')
    CAPACITY = (30, 'Capacity')


class PoolUnitTypeEnum(UnityEnum):
    RAID_GROUP = (1, 'RAID Group')
    VIRTUAL_DISK = (2, 'Virtual Disk')


class StoragePoolTypeEnum(UnityEnum):
    DYNAMIC = (1, 'Dynamic')
    TRADITIONAL = (2, 'Traditional')


class IpProtocolVersionEnum(UnityEnum):
    IPv4 = (4, 'IPv4')
    IPv6 = (6, 'IPv6')


class FileInterfaceRoleEnum(UnityEnum):
    PRODUCTION = (0, 'Production')
    BACKUP = (1, 'Backup')


class ReplicationPolicyEnum(UnityEnum):
    NOT_REPLICATED = (0, 'Not Replicated')
    Replicated = (1, 'Replicated')
    OVERRIDDEN = (2, 'Overridden')


class EnclosureTypeEnum(UnityEnum):
    DERRINGER_6G_SAS_DAE = (20, '25 Drive 6G DAE')
    PINECONE_6G_SAS_DAE = (26, '12 Drive 6G DAE')
    STEELJAW_6G_SAS_DPE = (27, '12 Drive 6G DPE')
    RAMHORN_6G_SAS_DPE = (28, '25 Drive 6G DPE')
    TABASCO_12G_SAS_DAE = (29, '25 Drive 12G DAE')
    ANCHO_12G_SAS_DAE = (30, '15 Drive 12G DAE')
    NAGA_12G_SAS_DAE = (32, '80 Drive 12G DAE')
    MIRANDA_12G_SAS_DPE = (36, '25 Drive 12G DPE')
    RHEA_12G_SAS_DPE = (37, '12 Drive 12G DPE')
    SCORP_12G_SAS_DPE = (38, '25 Drive 12G DPE')
    VIRTUAL_DPE = (100, 'Virtual DPE')
    UNSUPPORTED = (999, 'Unsupported Enclosure')


class DiskTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unsupported')
    SAS = (5, 'SAS')
    SAS_FLASH = (9, 'SAS Flash')
    NL_SAS = (10, 'Near-line SAS')
    SAS_FLASH_2 = (11, 'SAS Medium Endurance Flash')
    SAS_FLASH_3 = (12, 'SAS Low Endurance Flash')
    SAS_FLASH_4 = (13, 'SAS Low Endurance Flash')


class DiskTypeEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return DiskTypeEnum


class KdcTypeEnum(UnityEnum):
    CUSTOM = (0, 'Custom')
    UNIX = (1, 'Unix')
    WINDOWS = (2, 'Windows')


class ThinStatusEnum(UnityEnum):
    FALSE = (0, 'False')
    TRUE = (1, 'True')
    MIXED = (0xffff, 'Mixed')


class DedupStatusEnum(UnityEnum):
    DISABLED = (0, 'Disabled')
    ENABLED = (1, 'Enabled')
    MIXED = (0xffff, 'Mixed')


class ESXFilesystemMajorVersionEnum(UnityEnum):
    VMFS_3 = (3, 'VMFS 3')
    VMFS_5 = (5, 'VMFS 5')
    VMFS_6 = (6, 'VMFS 6')


class ESXFilesystemBlockSizeEnum(UnityEnum):
    _1MB = (1, '1 MB')
    _2MB = (2, '2 MB')
    _4MB = (4, '4 MB')
    _8MB = (8, '8 MB')


class ScheduleVersionEnum(UnityEnum):
    LEGACY = (1, 'Legacy Schedule')
    SIMPLE = (2, 'Simple Schedule')


class ScheduleTypeEnum(UnityEnum):
    N_HOURS_AT_MM = (0, 'Every N hours, at MM')
    DAY_AT_HHMM = (1, 'Each day at HH:MM')
    N_DAYS_AT_HHMM = (2, 'Every N days at HH:MM')
    SELDAYS_AT_HHMM = (3, 'On SEL days of week at HH:MM')
    NTH_DAYOFMONTH_AT_HHMM = (4, 'On Nth day of month at HH:MM')
    UNSUPPORTED = (5, 'Not supported')


class HostLUNAccessEnum(UnityEnum):
    NO_ACCESS = (0, 'No Access')
    PRODUCTION = (1, 'Production LUNs only')
    SNAPSHOT = (2, 'LUN Snapshots only')
    BOTH = (3, 'Production LUNs and Snapshots')
    # Applies to consistency groups only
    MIXED = (0xffff, 'Mixed')


class HostTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    HOST_MANUAL = (1, 'Manual Defined Host')
    SUBNET = (2, 'Hosts in a Subnet')
    NET_GROUP = (3, 'Net Group')
    RPA = (4, 'RecoverPoint Appliance')
    HOST_AUTO = (5, 'Auto-managed Host')
    VNX_SAN_COPY = (255, 'VNX Block Migration system')


class HostManageEnum(UnityEnum):
    UNKNOWN = (0, "Manged Manually")
    VMWARE = (1, 'Auto-managed by ESX Server')
    OTHERS = (2, 'Other Methods')


class HostRegistrationTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Registration Type Unknown')
    MANUAL = (1, 'Manually Registered Initiator')
    ESX_AUTO = (2, 'ESX Auto-registered Initiator')


class HostContainerTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    ESX = (1, 'ESX')
    VCENTER = (2, 'vCenter')


class HostInitiatorTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    FC = (1, 'FC')
    ISCSI = (2, 'iSCSI')


class HostInitiatorPathTypeEnum(UnityEnum):
    MANUAL = (0, 'Manual')
    ESX_AUTO = (1, 'ESX Auto')
    OTHER_AUTO = (2, 'Other Auto')


class HostInitiatorIscsiTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    HARDWARE = (1, 'Hardware')
    SOFTWARE = (2, 'Software')
    DEPENDENT = (3, 'Dependent')


class HostInitiatorSourceTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    HP_AUTO_TRESPASS = (2, 'HP with Auto-Trespass')
    OPEN_NATIVE = (3, 'Open Native')
    SGI = (9, 'Silicon Graphics')
    HP_NO_AUTO_TRESPASS = (10, 'HP without Auto-Traspass')
    DELL = (19, 'Dell')
    FUJITSU_SIEMENS = (22, 'Fujitsu-Siemens')
    CLARIION_ARRAY_CMI = (25, 'Remote CLARiiON array')
    TRU64 = (28, 'Tru64')
    RECOVER_POINT = (31, 'RecoverPoint')


class DatastoreTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    VMFS_3 = (1, 'VMFS3')
    VMFS_5 = (2, 'VMFS5')
    VMFS_6 = (3, 'VMFS6')


class VMDiskTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    VMFS_THICK = (1, 'VMFS Thick')
    VMFS_THIN = (2, 'VMFS Thin')
    RDM_PHYSICAL = (3, 'RDM Physical')
    RDM_VIRTUAL = (4, 'RDM Virtual')


class VMPowerStateEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OFF = (1, 'Off')
    ON = (2, 'On')
    SUSPENDED = (3, 'Suspended')
    PAUSED = (4, 'Paused')


class VVolDatastoreTypeEnum(UnityEnum):
    ISCSI = (0, 'iSCSI')
    NFS = (1, 'NFS')


class VVolTypeEnum(UnityEnum):
    CONFIG = (0, 'Config')
    DATA = (1, 'Data')
    SWAP = (2, 'Swap')
    MEMORY = (3, 'Memory')
    OTHER = (99, 'Other')


class ReplicaTypeEnum(UnityEnum):
    BASE = (0, 'Base vVol')
    PRE_SNAPSHOT = (1, 'Prepared Snapshot vVol')
    SNAPSHOT = (2, 'Snapshot vVol')
    FAST_CLONE = (3, 'Fast-clone vVol')


class HostPortTypeEnum(UnityEnum):
    IPv4 = (0, 'IPv4')
    IPv6 = (1, 'IPv6')
    NETWORK_NAME = (2, 'Network Name')


class HostLUNTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    LUN = (1, 'Production LUN')
    LUN_SNAP = (2, 'Snapshot LUN')


class FcSpeedEnum(UnityEnum):
    AUTO = (0, 'Auto')
    _1GbPS = (1, '1GbPS')
    _2GbPS = (2, '2GbPS')
    _4GbPS = (4, '4GbPS')
    _8GbPS = (8, '8GbPS')
    _16GbPS = (16, '16GbPS')
    _32GbPS = (32, '32GbPS')


class FcSpeedEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return FcSpeedEnum


class SFPSpeedValuesEnum(UnityEnum):
    AUTO = (0, 'Auto')
    _10MbPS = (10, '10MbPS')
    _100MbPS = (10, '100MbPS')
    _1GbPS = (1000, '1GbPS')
    _1500MbPS = (1500, '1500MbPS')
    _2GbPS = (2000, '2GbPS')
    _3GbPS = (3000, '3GbPS')
    _4GbPS = (4000, '4GbPS')
    _6GbPS = (6000, '6GbPS')
    _8GbPS = (8000, '8GbPS')
    _10GbPS = (10000, '10GbPS')
    _12GbPS = (12000, '12GbPS')
    _16GbPS = (16000, '16GbPS')
    _25GbPS = (25000, '25GbPS')
    _32GbPS = (32000, '32GbPS')
    _40GbPS = (40000, '40GbPS')
    _100GbPS = (100000, '100GbPS')
    _1TbPS = (1000000, '1TbPS')


class SFPSpeedValuesEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return SFPSpeedValuesEnum


class SFPProtocolValuesEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    FC = (1, 'FC')
    ETHERNET = (2, 'Ethernet')
    SAS = (3, 'SAS')


class SFPProtocolValuesEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return SFPProtocolValuesEnum


class ConnectorTypeEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    RJ45 = (1, 'RJ45')
    LC = (2, 'LC')
    MINI_SAS_HD = (3, 'MiniSAS HD')
    COPPER_PIGTAIL = (4, "Copper pigtail")
    NO_SEPARABLE_CONNECTOR = (5, "No separable connector")
    NAS_COPPER = (6, "NAS copper")
    NOT_PRESENT = (7, "Not present")


class SpeedValuesEnum(UnityEnum):
    _3Gbps = (3, "3Gbps")
    _6Gbps = (6, "6Gbps")
    _12Gbps = (12, "12Gbps")


class EPSpeedValuesEnum(UnityEnum):
    AUTO = (0, 'Auto')
    _10MbPS = (10, '10MbPS')
    _100MbPS = (100, '100MbPS')
    _1GbPS = (1000, '1GbPS')
    _10GbPS = (10000, '10GbPS')
    _25GbPS = (25000, '25GbPS')
    _40GbPS = (40000, '40GbPS')
    _100GbPS = (100000, '100GbPS')
    _1TbPS = (1000000, '1TbPS')


class EPSpeedValuesEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return EPSpeedValuesEnum


class DiskTierEnum(UnityEnum):
    EXTREME_PERFORMANCE = (0, 'Extreme Performance')
    PERFORMANCE = (1, 'Performance')
    CAPACITY = (2, 'Capacity')
    EXTREME_MULTI = (3, 'Multi-tier with Flash')
    MULTI = (4, 'Multi-tier without Flash')


class DiskTierEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return DiskTierEnum


class FastCacheStateEnum(UnityEnum):
    OFF = (0, 'Off')
    ON = (1, 'On')


class FastCacheStateEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return FastCacheStateEnum


class SpaceEfficiencyEnum(UnityEnum):
    THICK = (0, 'Thick')
    THIN = (1, 'Thin')


class SpaceEfficiencyEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return SpaceEfficiencyEnum


class ServiceLevelEnum(UnityEnum):
    BASIC = (0, 'Basic')
    BRONZE = (1, 'Bronze')
    SILVER = (2, 'Silver')
    GOLD = (3, 'Gold')
    PLATINUM = (4, 'PLATINUM')


class ServiceLevelEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return ServiceLevelEnum


class VmwarePETypeEnum(UnityEnum):
    NAS = (0, 'NAS')
    SCSI = (1, 'SCSI')


class NodeEnum(UnityEnum):
    SPA = (0, 'SPA')
    SPB = (1, 'SPB')
    UNKNOWN = (2989, 'Unknown')


class LUNTypeEnum(UnityEnum):
    GENERIC = (1, 'Generic Storage')
    STANDALONE = (2, 'Standalone Storage')
    VMWARE_ISCSI = (3, 'VMware Storage')


class JobStateEnum(UnityEnum):
    QUEUED = (1, 'Queued')
    RUNNING = (2, 'Running')
    SUSPENDED = (3, 'Suspended')
    COMPLETED = (4, 'Completed')
    FAILED = (5, 'Failed')
    ROLLING_BACK = (6, 'Rolling Back')
    COMPLETED_WITH_ERROR = (7, 'Completed with Error')


class JobTaskStateEnum(UnityEnum):
    NOT_STARTED = (0, 'Not Started')
    RUNNING = (1, 'Running')
    COMPLETED = (2, 'Completed')
    FAILED = (3, 'Failed')
    ROLLING_BACK = (5, 'Rolling Back')
    COMPLETED_WITH_PROBLEMS = (6, 'Completed With Errors')
    SUSPENDED = (7, 'Suspended')


class SeverityEnum(UnityEnum):
    OK = (8, 'OK')
    DEBUG = (7, 'Debug')
    INFO = (6, 'Info')
    NOTICE = (5, 'Notice')
    WARNING = (4, 'Warning')
    ERROR = (3, 'Error')
    CRITICAL = (2, 'Critical')
    ALERT = (1, 'Alert')
    EMERGENCY = (0, 'Emergency')


class ACEAccessTypeEnum(UnityEnum):
    DENY = (0, 'Deny')
    GRANT = (1, 'Grant')
    NONE = (2, 'None')


class ACEAccessLevelEnum(UnityEnum):
    READ = (1, 'Read')
    WRITE = (2, 'Write')
    FULL = (4, 'Full')


class IOLimitPolicyStateEnum(UnityEnum):
    GLOBAL_PAUSED = (1, 'Global Paused')
    PAUSED = (2, 'Paused')
    ACTIVE = (3, 'Active')


class IOLimitPolicyTypeEnum(UnityEnum):
    ABSOLUTE = (1, 'Absolute Value')
    DENSITY_BASED = (2, 'Density-based Value')


class DNSServerOriginEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    STATIC = (1, 'Set Manually')
    DHCP = (2, 'Configured by DHCP')


class MetricTypeEnum(UnityEnum):
    UNKNOWN = (1, 'Unknown')
    COUNTER_32 = (2, '32 bits Counter')
    COUNTER_64 = (3, '64 bits Counter')
    RATE = (4, 'Rate')
    FACT = (5, 'Fact')
    TEXT = (6, 'Text')
    VIRTUAL_COUNTER_32 = (7, '32 bits Virtual Counter')
    VIRTUAL_COUNTER_64 = (8, '64 bits Virtual Counter')


class DiskTechnologyEnum(UnityEnum):
    SAS = (1, 'SAS')
    NL_SAS = (2, 'NL_SAS')
    EFD = (5, 'EFD')
    SAS_FLASH_2 = (6, 'SAS_FLASH_2')
    SAS_FLASH_3 = (7, 'SAS_FLASH_3')
    SAS_FLASH_4 = (8, 'SAS_FLASH_4')
    SAS_FLASH_5 = (9, 'SAS_FLASH_5')
    MIXED = (50, 'Mixed')
    VIRTUAL = (99, 'Virtual')


class ThinCloneActionEnum(UnityEnum):
    DD_COPY = (1, 'Copy via dd')
    LUN_ATTACH = (2, 'Attach a base LUN')
    TC_DELETE = (3, 'Delete a thin-cloned LUN')
    BASE_LUN_DELETE = (4, 'Delete the base LUN of a thin-cloned LUN')


class FeatureStateEnum(UnityEnum):
    Disabled = (1, "FeatureStateDisabled")
    Enabled = (2, "FeatureStateEnabled")
    Hidden = (3, "FeatureStateHidden")


class FeatureReasonEnum(UnityEnum):
    UNLICENSED = (1, "FeatureReasonUnlicensed")
    EXPIRED_LICENSE = (2, "FeatureReasonExpiredLicense")
    PLATFORM_RESTRICTION = (3, "FeatureReasonPlatformRestriction")
    EXCLUDED = (4, "FeatureReasonExcluded")


class HotSparePolicyStatusEnum(UnityEnum):
    OK = (0, "OK")
    VIOLATED = (741, "Violated")


class InterfaceConfigModeEnum(UnityEnum):
    DISABLED = (0, "Disabled")
    STATIC = (1, "Static")
    AUTO = (2, "Auto")


class MoveSessionStateEnum(UnityEnum):
    INITIALIZING = (0, "Initializing")
    QUEUED = (1, "Queued")
    RUNNING = (2, "Running")
    FAILED = (3, "Failed")
    CANCELLING = (4, "Cancelling")
    CANCELLED = (5, "Cancelled")
    COMPLETED = (6, "Completed")


class MoveSessionStatusEnum(UnityEnum):
    INITIALIZING = (0, "OK")
    POOL_OFFLINE = (1, "PoolOffline")
    POOL_OUT_OF_SPACE = (2, "PoolOutOfSpace")
    INTERNAL_ERROR = (3, "InternalError")


class MoveSessionPriorityEnum(UnityEnum):
    IDLE = (0, "Idle")
    LOW = (1, "Low")
    BELOW_NORMAL = (2, "Below_Normal")
    NORMAL = (3, "Normal")
    ABOVE_NORMAL = (4, "Above_Normal")
    HIGH = (5, "High")


class ReplicationSessionStatusEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OTHER = (1, 'Other')
    OK = (2, 'OK')
    PAUSED = (3, 'Paused')
    FATAL_REPLICATION_ISSUE = (4, 'Fatal_Replication_Issue')
    LOST_COMMUNICATION = (5, 'Lost_Communication')
    FAILED_OVER = (6, 'Failed_Over')
    FAILED_OVER_WITH_SYNC = (7, 'Failed_Over_With_Sync')
    DESTINATION_EXTEND_NOT_SYNCING = (8, 'Destination_Extend_Not_Syncing')
    DESTINATION_EXTEND_IN_PROGRESS = (9, 'Destination_Extend_In_Progress')
    LOST_SYNC_COMMUNICATION = (10, 'Lost_Sync_Communication')
    DESTINATION_POOL_OUT_OF_SPACE = (11, 'Destination_Pool_Out_Of_Space')


class ReplicationSessionNetworkStatusEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OTHER = (1, 'Other')
    OK = (2, 'OK')
    LOST_COMMUNICATION = (5, 'Lost_Communication')
    LOST_SYNC_COMMUNICATION = (10, 'Lost_Sync_Communication')


class ReplicationSessionSyncStateEnum(UnityEnum):
    MANUAL_SYNCING = (0, 'Manual_Syncing')
    AUTO_SYNCING = (1, 'Auto_Syncing')
    IDLE = (2, 'Idle')
    UNKNOWN = (100, 'Unknown')
    OUT_OF_SYNC = (101, 'Out_Of_Sync')
    IN_SYNC = (102, 'In_Sync')
    CONSISTENT = (103, 'Consistent')
    SYNCING = (104, 'Syncing')
    INCONSISTENT = (105, 'Inconsistent')


class ReplicationSessionReplicationRoleEnum(UnityEnum):
    SOURCE = (0, 'Source')
    DESTINATION = (1, 'Destination')
    LOOPBACK = (2, 'Loopback')
    LOCAL = (3, 'Local')
    UNKNOWN = (4, 'Unknown')


class ReplicationCapabilityEnum(UnityEnum):
    SYNC = (0, 'Sync')
    ASYNC = (1, 'Async')
    BOTH = (2, 'Both')
    NONE = (3, 'None')


class ReplicationEndpointResourceTypeEnum(UnityEnum):
    FILESYSTEM = (1, 'filesystem')
    CONSISTENCYGROUP = (2, 'consistencyGroup')
    VMWAREFS = (3, 'vmwarefs')
    VMWAREISCSI = (4, 'vmwareiscsi')
    LUN = (8, 'lun')
    NASSERVER = (10, 'nasServer')


class ReplicationOpStatusEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OTHER = (1, 'Other')
    OK = (2, 'OK')
    NON_RECOVERABLE_ERROR = (7, 'Non_Recoverable_Error')
    LOST_COMMUNICATION = (13, 'Lost_Communication')
    FAILED_OVER_WITH_SYNC = (0x8400, 'Failed_Over_with_Sync')
    FAILED_OVER = (0x8401, 'Failed_Over')
    MANUAL_SYNCING = (0x8402, 'Manual_Syncing')
    PAUSED = (0x8403, 'Paused')
    IDLE = (0x8404, 'Idle')
    AUTO_SYNC_CONFIGURED = (0x8405, 'Auto_Sync_Configured')
    DESTINATION_EXTEND_FAILED_NOT_SYNCING = (
        0x840B, 'Destination_Extend_Failed_Not_Syncing')
    DESTINATION_EXTEND_IN_PROGRESS = (0x840C, 'Destination_Extend_In_Progress')
    ACTIVE = (0x840D, 'Active')
    LOST_SYNC_COMMUNICATION = (0x840E, 'Lost_Sync_Communication')
    DESTINATION_POOL_OUT_OF_SPACE = (0x840F, 'Destination_Pool_Out_Of_Space')
    SYNCING = (0x8411, 'Syncing')
    FAILED_OVER_WITH_SYNC_MIXED = (0x87E8, 'Failed_Over_with_Sync_Mixed')
    FAILED_OVER_MIXED = (0x87E9, 'Failed_Over_Mixed')
    MANUAL_SYNCING_MIXED = (0x87EA, 'Manual_Syncing_Mixed')
    PAUSED_MIXED = (0x87EB, 'Paused_Mixed')
    IDLE_MIXED = (0x87EC, 'Idle_Mixed')
    AUTO_SYNC_CONFIGURED_MIXED = (0x87ED, 'Auto_Sync_Configured_Mixed')


class SNMPAuthProtocolEnum(UnityEnum):
    NONE = (0, 'None')
    MD5 = (1, 'MD5')
    SHA = (2, 'SHA')


class SNMPPrivacyProtocolEnum(UnityEnum):
    NONE = (0, 'None')
    AES = (1, 'AES')
    DES = (2, 'DES')


class SNMPVersionEnum(UnityEnum):
    V1 = (1, 'v1')
    V2C = (2, 'v2c')
    V3 = (3, 'v3')


class RemoteObjectTypeEnum(UnityEnum):
    LUN = (0, 'lun')
    FILESYSTEM = (1, 'filesystem')
    CONSISTENCYGROUP = (2, 'consistencyGroup')
    NASSERVER = (3, 'nasServer')


class ImportUnixDirectoryServiceEnum(UnityEnum):
    LOCAL = (0, 'Local')
    NIS = (1, 'NIS')
    LDAP = (2, 'LDAP')
    LOCALTHENNIS = (3, 'LocalThenNis')
    LOCALTHENLDAP = (4, 'LocalThenLdap')
    NONE = (5, 'None')
    DIRECTMATCH = (6, 'DirectMatch')


class ImportUnixDirectoryServiceEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return ImportUnixDirectoryServiceEnum


class ImportCapabilityEnum(UnityEnum):
    IMPORTABLE = (0, 'Importable')
    NON_IMPORTABLE_INTERNAL_ERROR = (1, 'Non_importable_internal_error')
    NON_IMPORTABLE_ONLY_SUPPORT_VNX = (2, 'Non_importable_only_support_vnx')
    NON_IMPORTABLE_SRC_VDM_NAME_IN_USE_SESS_COMPLETED = (
        3, 'Non_importable_src_vdm_name_in_use_sess_completed')
    NON_IMPORTABLE_SRC_VDM_NAME_IN_USE_SESS_ACTIVE = (
        4, 'Non_importable_src_vdm_name_in_use_sess_active')
    NON_IMPORTABLE_SRC_VDM_NAME_IN_USE = (
        5, 'Non_importable_src_vdm_name_in_use')
    NON_IMPORTABLE_NO_UP_SRC_CLNT_IF = (6, 'Non_importable_no_up_src_clnt_if')
    NON_IMPORTABLE_NO_FS_ON_SRC_VDM = (7, 'Non_importable_no_fs_on_src_vdm')
    NON_IMPORTABLE_REACH_SRC_FS_LIMIT = (
        8, 'Non_importable_reach_src_fs_limit')
    NON_IMPORTABLE_NFSV4_NOT_SUPPORT = (9, 'Non_importable_nfsv4_not_support')
    NON_IMPORTABLE_SECNFS_NOT_SUPPORT = (
        10, 'Non_importable_secnfs_not_support')
    NON_IMPORTABLE_VDM_CANNOT_HAVE_CIFS_SERVER = (
        11, 'Non_importable_vdm_cannot_have_cifs_server')
    NON_IMPORTABLE_SRCMIGIF_MORE_ERROR = (
        12, 'Non_importable_srcmigif_more_error')
    NON_IMPORTABLE_SRCMIGIF_ERROR = (13, 'Non_importable_srcmigif_error')
    NON_IMPORTABLE_INVALID_VDM = (14, 'Non_importable_invalid_vdm')
    NON_IMPORTABLE_REACH_SRC_IF_LIMIT = (
        15, 'Non_importable_reach_src_if_limit')
    NON_IMPORTABLE_SRC_CS_NOT_PRIMARY = (
        16, 'Non_importable_src_cs_not_primary')


class VDMSupportedProtocolsEnum(UnityEnum):
    NFS = (0, 'NFS')
    CIFS = (1, 'CIFS')
    MULTIPROTOCOL = (2, 'Multiprotocol')
    CIFSANDNFS = (3, 'CIFSAndNFS')


class ImportStateEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    INITIALIZED = (50000, 'Initialized')
    INITIAL_COPY = (50001, 'Initial_Copy')
    READY_TO_CUTOVER = (50002, 'Ready_to_Cutover')
    PAUSED = (50003, 'Paused')
    CUTTING_OVER = (50004, 'Cutting_Over')
    INCREMENTAL_COPY = (50005, 'Incremental_Copy')
    READY_TO_COMMIT = (50006, 'Ready_to_Commit')
    COMMITTING = (50007, 'Committing')
    COMPLETED = (50008, 'Completed')
    CANCELLING = (50009, 'Cancelling')
    CANCELLED = (50010, 'Cancelled')
    PENDING = (50011, 'Pending')
    SYNCING = (50012, 'Syncing')
    ERROR = (50013, 'Error')


class ImportOpStatusEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    NON_RECOVERABLE_ERROR = (7, 'Non_Recoverable_Error')
    CONFIGURING_TARGET_RESOURCE = (32768, 'Configuring_Target_Resource')
    ENABLING_TARGET_RESOURCE_ACCESS = (
        32769, 'Enabling_Target_Resource_Access')
    MIGRATOR_CANNOT_DISCOVER_TARGET_RESOURCE = (
        32770, 'Migrator_Cannot_Discover_Target_Resource')
    READY_TO_MIGRATE = (32771, 'Ready_To_Migrate')
    INITIAL_TRANSFER_INPROGRESS = (32772, 'Initial_Transfer_Inprogress')
    DELTA_TRANSFER_INPROGRESS = (32773, 'Delta_Transfer_Inprogress')
    PAUSED = (32774, 'Paused')
    READY_TO_CUTOVER = (32776, 'Ready_To_Cutover')
    LOST_COMMUNICATION = (32777, 'Lost_Communication')
    TARGET_LUNS_OFFLINE = (32779, 'Target_Luns_Offline')
    PENDING = (32780, 'Pending')
    CUTOVER_SYNC_COMPLETE = (32781, 'Cutover_Sync_Complete')
    SESSION_CLEANUP_COMPLETE = (32782, 'Session_Cleanup_Complete')
    FINAL_TRANSFER_INPROGRESS = (32783, 'Final_Transfer_Inprogress')
    INITIALIZED = (33024, 'Initialized')
    STARTING = (33025, 'Starting')
    START_FAILED = (33026, 'Start_Failed')
    MIGRATING_DATA = (33027, 'Migrating_Data')
    MIGRATING_DATA_STOPPED = (33028, 'Migrating_Data_Stopped')
    MIGRATING_DATA_FAILED = (33029, 'Migrating_Data_Failed')
    MIGRATING_DATA_FAILED_STOPPED = (33030, 'Migrating_Data_Failed_Stopped')
    MIGRATING_CONFIGURATION = (33031, 'Migrating_Configuration')
    MIGRATING_CONFIGURATION_FAILED = (33032, 'Migrating_Configuration_Failed')
    MIGRATING_CONFIGURATION_PAUSED = (33033, 'Migrating_Configuration_Paused')
    CUTTING_OVER = (33280, 'Cutting_Over')
    CUTOVER_FAILED = (33281, 'Cutover_Failed')
    SYNCING_DATA = (33282, 'Syncing_Data')
    SYNCING_DATA_STOPPED = (33283, 'Syncing_Data_Stopped')
    SYNCING_DATA_FAILED = (33284, 'Syncing_Data_Failed')
    SYNCING_DATA_FAILED_STOPPED = (33285, 'Syncing_Data_Failed_Stopped')
    READY_TO_COMPLETE = (33536, 'Ready_To_Complete')
    COMPLETING = (33537, 'Completing')
    COMPLETE_FAILED = (33538, 'Complete_Failed')
    COMPLETED = (33539, 'Completed')
    CANCELLING = (33540, 'Cancelling')
    CANCEL_FAILED = (33541, 'Cancel_Failed')
    CANCELLED = (33542, 'Cancelled')
    MIGRATING_DATA_STOPPING = (33543, 'Migrating_Data_Stopping')
    SYNCING_DATA_STOPPING = (33544, 'Syncing_Data_Stopping')
    PROVISIONING_TARGET_PAUSED = (33547, 'Provisioning_Target_Paused')
    FS_OK = (34048, 'FS_OK')
    FS_SOURCE_IO_FAILURE = (34049, 'FS_Source_IO_Failure')
    FS_DESTINATION_IO_FAILURE = (34050, 'FS_Destination_IO_Failure')
    FS_CONNECTION_FAILURE = (34051, 'FS_Connection_Failure')
    FS_UNRECOVERABLE_FAILURE = (34052, 'FS_Unrecoverable_Failure')
    VMO_FAULTED = (34304, 'VMO_Faulted')
    VMO_OFFLINE = (34305, 'VMO_Offline')
    ELEMENT_IMPORT_OK = (34560, 'Element_Import_OK')
    ELEMENT_IMPORT_UNABLE_TO_LOCATE_DEVICE = (
        34561, 'Element_Import_Unable_To_Locate_Device')
    ELEMENT_IMPORT_BAD_BLOCK_ON_SOURCE_DEVICE = (
        34562, 'Element_Import_Bad_Block_On_Source_Device')
    ELEMENT_IMPORT_UNABLE_TO_ACCESS_DEVICE = (
        34563, 'Element_Import_Unable_To_Access_Device')
    ELEMENT_IMPORT_LU_TRESPASSED = (34564, 'Element_Import_LU_Trespassed')
    ELEMENT_IMPORT_SOURCE_DEVICE_INACCESSIBLE = (
        34565, 'Element_Import_Source_Device_Inaccessible')
    ELEMENT_IMPORT_LOW_USER_LINK_BANDWIDTH = (
        34566, 'Element_Import_Low_User_Link_Bandwidth')
    ELEMENT_IMPORT_CONCURRENT_SANCOPY_SESSION_DESTINATIONS = (
        34567, 'Element_Import_Concurrent_SanCopy_Session_Destinations')
    ELEMENT_IMPORT_ERROR_COMMUNICATING_WITH_SANPVIEW = (
        34568, 'Element_Import_Error_Communicating_With_SanpView')
    ELEMENT_IMPORT_ERROR_COMMUNICATING_WITH_SANPVIEW_1 = (
        34569, 'Element_Import_Error_Communicating_With_SanpView_1')
    ELEMENT_IMPORT_SESSION_INCONSISTENT_STATE = (
        34570, 'Element_Import_Session_Inconsistent_State')
    ELEMENT_IMPORT_DESTINATION_INCONSISTENT_STATE = (
        34571, 'Element_Import_Destination_Inconsistent_State')
    ELEMENT_IMPORT_AUTO_RECOVERY_RESUME_FAILED = (
        34572, 'Element_Import_Auto_Recovery_Resume_Failed')
    ELEMENT_IMPORT_ALL_PATHS_FAILURE = (
        34573, 'Element_Import_All_Paths_failure')
    ELEMENT_IMPORT_ACCESS_DENIED_TO_DEVICE = (
        34574, 'Element_Import_Access_Denied_To_Device')
    ELEMENT_IMPORT_NOT_ENOUGH_MEMORY = (
        34575, 'Element_Import_Not_Enough_Memory')
    ELEMENT_IMPORT_SOURCE_DEVICE_FAILURE = (
        34576, 'Element_Import_Source_Device_Failure')
    ELEMENT_IMPORT_DESTINATION_DEVICE_FAILURE = (
        34577, 'Element_Import_Destination_Device_Failure')
    ELEMENT_IMPORT_DESTINATION_DEVICE_NOT_FOUND = (
        34578, 'Element_Import_Destination_Device_Not_found')
    ELEMENT_IMPORT_TARGET_LU_NOT_INITIALIZED = (
        34579, 'Element_Import_Target_LU_Not_Initialized')
    ELEMENT_IMPORT_COMMAND_TIMEDOUT = (
        34580, 'Element_Import_Command_TimedOut')
    ELEMENT_IMPORT_VERIFYING_FRONTEND_TIMEDOUT = (
        34581, 'Element_Import_Verifying_Frontend_TimedOut')
    ELEMENT_IMPORT_VERIFYING_FRONTEND_TIMEDOUT_ANOTHER_OPERATION = (
        34582, 'Element_Import_Verifying_Frontend_TimedOut_Another_Operation')
    ELEMENT_IMPORT_SOURCE_CONNECTIVITY_TIMEDOUT = (
        34583, 'Element_Import_Source_Connectivity_TimedOut')
    ELEMENT_IMPORT_DESTINATION_CONNECTIVITY_TIMEDOUT = (
        34584, 'Element_Import_Destination_Connectivity_TimedOut')
    ELEMENT_IMPORT_RLP_IO_FAILURE = (34585, 'Element_Import_RLP_IO_Failure')
    ELEMENT_IMPORT_TOTAL_SESSIONS_LIMIT_REACHED = (
        34586, 'Element_Import_Total_Sessions_Limit_Reached')
    ELEMENT_IMPORT_INCREMENTAL_SESSIONS_LIMIT_REACHED = (
        34587, 'Element_Import_Incremental_Sessions_Limit_Reached')
    ELEMENT_IMPORT_INCREMENTAL_SESSIONS_TOTAL_NUMBER_REACHED = (
        34588, 'Element_Import_Incremental_Sessions_Total_Number_Reached')
    ELEMENT_IMPORT_LIMIT_OF_TOTAL_SESSIONS_REACHED = (
        34589, 'Element_Import_Limit_Of_Total_Sessions_Reached')
    ELEMENT_IMPORT_LIMIT_OF_TOTAL_INCREMENTAL_SESSIONS_REACHED = (
        34590, 'Element_Import_Limit_Of_Total_Incremental_Sessions_Reached')
    ELEMENT_IMPORT_COPY_COMMAND_QUEUED = (
        34591, 'Element_Import_Copy_Command_Queued')
    ELEMENT_IMPORT_SESSION_FAILED_ON_SOURCE_OR_DESTINATION = (
        34592, 'Element_Import_Session_Failed_On_Source_Or_Destination')
    ELEMENT_IMPORT_DEVICE_CANNOT_BE_LOCATED = (
        34593, 'Element_Import_Device_Cannot_Be_Located')
    ELEMENT_IMPORT_NO_UNUSED_RLP_LUNS = (
        34594, 'Element_Import_No_Unused_Rlp_Luns')
    ELEMENT_IMPORT_RESERVED_LUN_NOT_SUPPORT_INCREMENTAL_SESSIONS = (
        34595, 'Element_Import_Reserved_Lun_Not_Support_Incremental_Sessions')
    ELEMENT_IMPORT_SNAPVIEW_RESERVED_LUN_NOT_ENOUGH_SPACE = (
        34596, 'Element_Import_Snapview_Reserved_Lun_Not_Enough_Space')
    ELEMENT_IMPORT_TOO_MANY_SNAPSHOTS_ON_SOURCE_LU = (
        34597, 'Element_Import_Too_Many_Snapshots_On_Source_Lu')
    ELEMENT_IMPORT_CANNOT_OPEN_RESERVED_LUN = (
        34598, 'Element_Import_Cannot_Open_Reserved_Lun')
    ELEMENT_IMPORT_CANNOT_GET_RESERVED_LUN_INFO = (
        34599, 'Element_Import_Cannot_Get_Reserved_Lun_info')
    ELEMENT_IMPORT_NO_SPACE_ON_RLP = (34600, 'Element_Import_No_Space_On_Rlp')
    ELEMENT_IMPORT_RLP_MAXIMUM_DEVICES = (
        34601, 'Element_Import_Rlp_Maximum_Devices')
    ELEMENT_IMPORT_SESSION_WITH_NO_CACHE_DEVICES = (
        34602, 'Element_Import_Session_With_No_Cache_Devices')
    ELEMENT_IMPORT_SESSION_FAILED_WRITE_TO_TARGET_INSUFFICIENT_STORAGE = (
        34603,
        'Element_Import_Session_Failed_Write_To_Target_Device_'
        'Insufficient_Storage')
    ELEMENT_IMPORT_SESSION_DEVICE_NOT_READY = (
        34604, 'Element_Import_Session_Device_Not_Ready')
    ELEMENT_IMPORT_SESSION_SOURCE_DEVICE_UNAVAILABLE = (
        34605, 'Element_Import_Session_Source_Device_Unavailable')
    ELEMENT_IMPORT_SESSION_SOURCE_IN_IMPORT_SESSION = (
        34606, 'Element_Import_Session_Source_In_Import_Session')


class ImportOpStatusEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return ImportOpStatusEnum


class ImportTypeEnum(UnityEnum):
    BLOCK = (0, 'block')
    NAS = (1, 'nas')


class PoolConsumerTypeEnum(UnityEnum):
    FILESYSTEM = (1, 'FileSystem')
    CONSISTENCYGROUP = (2, 'ConsistencyGroup')
    VMWARENFS = (3, 'VMwareNFS')
    VMWAREVMFS = (4, 'VMwareVMFS')
    LUN = (8, 'LUN')
    VVOLDATASTOREFS = (9, 'VVolDatastoreFS')
    VVOLDATASTOREISCSI = (10, 'VVolDatastoreISCSI')
    NASSERVER = (32768, 'NASServer')


class ImportStageEnum(UnityEnum):
    INITIAL = (0, 'Initial')
    INCREMENTAL = (1, 'Incremental')
    FINAL = (2, 'Final')


class DayOfWeekEnum(UnityEnum):
    SUNDAY = (1, 'Sunday')
    MONDAY = (2, 'Monday')
    TUESDAY = (3, 'Tuesday')
    WEDNESDAY = (4, 'Wednesday')
    THURSDAY = (5, 'Thursday')
    FRIDAY = (6, 'Friday')
    SATURDAY = (7, 'Saturday')


class DayOfWeekEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return DayOfWeekEnum


class HostOperationalStatusEnum(UnityEnum):
    UNKNOWN = (0, 'Unknown')
    OTHER = (1, 'Other')
    OK = (2, 'OK')
    DEGRADED = (3, 'Degraded')
    STRESSED = (4, 'Stressed')
    PREDICTIVE_FAILURE = (5, 'Predictive_Failure')
    ERROR = (6, 'Error')
    NON_RECOVERABLE_ERROR = (7, 'Non_Recoverable_Error')
    STARTING = (8, 'Starting')
    STOPPING = (9, 'Stopping')
    STOPPED = (10, 'Stopped')
    IN_SERVICE = (11, 'In_Service')
    NO_CONTACT = (12, 'No_Contact')
    LOST_COMMUNICATION = (13, 'Lost_Communication')
    ABORTED = (14, 'Aborted')
    DORMANT = (15, 'Dormant')
    SUPPORTING_ENTITY_IN_ERROR = (16, 'Supporting_Entity_in_Error')
    COMPLETED = (17, 'Completed')
    POWER_MODE = (18, 'Power_Mode')
    TEMP_UNMOUNTED = (19, 'Temp_Unmounted')
    PERM_UNMOUNTED = (20, 'Perm_Unmounted')
    INITIATORS_CONFIG_NOT_HA = (32768, 'Initiators_Config_not_HA')
    NO_LOGGEDIN_INITIATOR = (32769, 'No_LoggedIn_Initiator')
    INITIATORS_DATA_INCONSISTENT = (32770, 'Initiators_Data_Inconsistent')
    NETWORK_ADDRESSES_DATA_INCONSISTENT = (
        32771, 'Network_Addresses_Data_Inconsistent')
    TENANT_DATA_INCONSISTENT = (32772, 'Tenant_Data_Inconsistent')
    CREDENTIAL_INVALID = (34048, 'Credential_Invalid')
    STORAGE_NOT_FOUND = (34049, 'Storage_Not_Found')
    CERTIFICATE_INVALID = (34050, 'Certificate_Invalid')


class HostOperationalStatusEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return HostOperationalStatusEnum


class VasaProviderStateEnum(UnityEnum):
    NOT_REGISTERED = (0, 'Not_Registered')
    REGISTERED = (1, 'Registered')
    NOT_SUPPORTED = (99, 'Not_Supported')


class HostContainerPotentialHostMatchConditionEnum(UnityEnum):
    MATCH_UNKNOWN = (0, 'Match_Unknown')
    MATCH_UUID = (1, 'Match_UUID')
    MATCH_ALLINITIATORS = (2, 'Match_AllInitiators')
    MATCH_UUID_ALLINITIATORS = (3, 'Match_UUID_AllInitiators')
    MATCH_ALLNETWORKADDRESSES = (4, 'Match_AllNetworkAddresses')
    MATCH_UUID_ALLNETWORKADDRESSES = (5, 'Match_UUID_AllNetworkAddresses')
    MATCH_ALLINITIATORS_ALLNETWORKADDRESSES = (
        6, 'Match_AllInitiators_AllNetworkAddresses')
    MATCH_UUID_ALLINITIATORS_ALLNETWORKADDRESSES = (
        7, 'Match_UUID_AllInitiators_AllNetworkAddresses')
    MATCH_SOMEINITIATORS = (8, 'Match_SomeInitiators')
    MATCH_UUID_SOMEINITIATORS = (9, 'Match_UUID_SomeInitiators')
    MATCH_ALLNETWORKADDRESSES_SOMEINITIATORS = (
        12, 'Match_AllNetworkAddresses_SomeInitiators')
    MATCH_UUID_ALLNETWORKADDRESSES_SOMEINITIATORS = (
        13, 'Match_UUID_AllNetworkAddresses_SomeInitiators')
    MATCH_SOMENETWORKADDRESSES = (16, 'Match_SomeNetworkAddresses')
    MATCH_UUID_SOMENETWORKADDRESSES = (17, 'Match_UUID_SomeNetworkAddresses')
    MATCH_ALLINITIATORS_SOMENETWORKADDRESSES = (
        18, 'Match_AllInitiators_SomeNetworkAddresses')
    MATCH_UUID_ALLINITIATORS_SOMENETWORKADDRESSES = (
        19, 'Match_UUID_AllInitiators_SomeNetworkAddresses')
    MATCH_SOMEINITIATORS_SOMENETWORKADDRESSES = (
        24, 'Match_SomeInitiators_SomeNetworkAddresses')
    MATCH_UUID_SOMEINITIATORS_SOMENETWORKADDRESSES = (
        25, 'Match_UUID_SomeInitiators_SomeNetworkAddresses')


class HostContainerPotentialHostMatchConditionEnumList(UnityEnumList):
    @classmethod
    def get_enum_class(cls):
        return HostContainerPotentialHostMatchConditionEnum


class HostContainerPotentialHostImportOptionEnum(UnityEnum):
    IMPORT_NEW = (0, 'Import_New')
    IMPORT_EXISTING = (1, 'Import_Existing')
    IMPORT_CONFLICT = (2, 'Import_Conflict')
    IMPORT_MATCH = (3, 'Import_Match')
    IMPORT_WARN = (4, 'Import_Warn')



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

import time
from hamcrest import assert_that, equal_to, ends_with, contains_string, \
    greater_than, less_than_or_equal_to, greater_than_or_equal_to, raises, \
    has_items

from storops_test.vnx.cli_mock import patch_cli
from storops.exception import VNXSystemDownError, VNXCredentialError
from storops.vnx.heart_beat import NodeInfo, NodeHeartBeat

__author__ = 'Cedric Zhuang'


class NodeHeartBeatTest(TestCase):
    def get_test_hb(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1')
        hb.add('spb', '1.1.1.2')
        return hb

    @patch_cli
    def test_normal(self):
        hb = NodeHeartBeat(interval=0.2)
        hb.add('spa', '1.1.1.1')
        hb.add('spb', '1.1.1.2')
        assert_that(hb.is_available('spa'), equal_to(True))
        assert_that(hb.is_available('spb'), equal_to(True))
        time.sleep(0.5)
        assert_that(hb.is_available('spa'), equal_to(True))
        assert_that(hb.is_available('spb'), equal_to(True))
        hb.stop()
        time.sleep(0.5)

    def test_add(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1')
        hb.add('cs', None)
        assert_that(hb.is_available('cs'), equal_to(False))

    def test_remove_extra_quotes(self):
        info = NodeInfo('spa', '  \'"1.1.1.1" \'   ')
        assert_that(info.ip, equal_to('1.1.1.1'))

    def test_is_updated_within(self):
        info = NodeInfo('spa', '1.1.1.1')
        info.available = True
        assert_that(info.is_updated_within(0), equal_to(False))
        assert_that(info.is_updated_within(1), equal_to(True))

    @patch_cli
    def test_get_alive_sp_ip(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1', False)
        hb.add('spb', '1.1.1.2', True)
        assert_that(hb.get_alive_sp_ip(), equal_to('1.1.1.2'))

    @patch_cli
    def test_get_alive_sp_ip_none(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1', False, working=False)
        hb.add('spb', '1.1.1.2', None, working=True)
        assert_that(hb.get_alive_sp_ip(), '1.1.1.2')

    @patch_cli
    def test_get_alive_sp_ip_down(self):
        def f():
            hb = NodeHeartBeat(interval=0)
            hb.add('spa', '1.1.1.1', False)
            hb.add('spb', '1.1.1.2', False)
            hb.get_alive_sp_ip()

        assert_that(f, raises(VNXSystemDownError,
                              'no storage processor available'))

    @patch_cli
    def test_get_alive_sp_ip_no_load_balance(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1', True, True)
        hb.add('spb', '1.1.1.2', True, False)
        assert_that(hb.get_alive_sp_ip(), equal_to('1.1.1.1'))

    @patch_cli
    def test_update_by_ip(self):
        hb = NodeHeartBeat(interval=0)
        hb.add('spa', '1.1.1.1', False)
        hb.add('spb', '1.1.1.2', False)
        hb.update_by_ip('1.1.1.1', True)
        assert_that(hb.get_alive_sp_ip(), equal_to('1.1.1.1'))

    @patch_cli
    def test_update_by_ip_latency(self):
        hb = NodeHeartBeat(interval=0.01)
        hb.add('spa', '1.1.1.1', False)
        node = hb.nodes[0]
        assert_that(node.available, equal_to(False))
        time.sleep(0.06)
        assert_that(node.available, equal_to(True))
        assert_that(hb.command_count, greater_than(1))
        hb.stop()
        time.sleep(0.06)

    @patch_cli
    def test_interval_change(self):
        hb = NodeHeartBeat(interval=0.1)
        hb.add('spa', '1.1.1.1')
        hb.add('spb', '1.1.1.2')
        time.sleep(0.6)
        hb.interval = 2
        time.sleep(1)
        # total call count is calculated like this:
        #   interval = 0.1, duration = 0.5, 5 >= cycle >= 4
        #   interval = 2, duration = 1, 0 <= cycle <= 1
        #   4 <= total cycle <= 6
        # each cycle has 2 call (one for each SP)
        # 8 <= total call count <= 12
        assert_that(hb.command_count, less_than_or_equal_to(12))
        assert_that(hb.command_count, greater_than_or_equal_to(4))
        hb.stop()
        time.sleep(1)

    @patch_cli
    def test_interval_no_loop(self):
        hb = self.get_test_hb()
        assert_that(hb.command_count, equal_to(0))
        hb.interval = 1
        time.sleep(0.2)
        assert_that(hb.command_count, greater_than(0))

    @patch_cli
    def test_repr(self):
        hb = self.get_test_hb()
        string = repr(hb)
        assert_that(string, contains_string('node count: 2'))

    def test_ping_sp_cmd(self):
        hb = NodeHeartBeat(interval=20)
        ip = '1.1.1.1'
        hb.add('spa', ip)
        hb.update_by_ip(ip, latency=5.123)
        assert_that(' '.join(hb.get_agent(ip)),
                    ends_with('-h 1.1.1.1 -t 35 -np getagent'))
        hb.stop()

    def test_is_all_sps_alive(self):
        hb = self.get_test_hb()
        assert_that(hb.is_all_sps_alive(), equal_to(True))
        hb.update_by_ip('1.1.1.1', available=False)
        assert_that(hb.is_all_sps_alive(), equal_to(False))

    def test_get_all_alive_sps_ip(self):
        hb = self.get_test_hb()
        assert_that(hb.get_all_alive_sps_ip(),
                    has_items('1.1.1.1', '1.1.1.2'))

    @patch_cli(output='credential_error.txt')
    def test_execute_cmd_credential_error(self):
        hb = self.get_test_hb()
        try:
            hb.execute_cmd(
                '1.1.1.1',
                ['naviseccli', '-h', '1.1.1.1', 'getagent'])
        except VNXCredentialError:
            pass
        assert_that(hb.is_credential_valid, equal_to(False))

    @patch_cli(output='ip_error.txt')
    def test_execute_cmd_ip_error(self):
        def f():
            hb = self.get_test_hb()
            hb.execute_cmd('abc', ['naviseccli', '-h', 'abc', 'getagent'])

        assert_that(f, raises(VNXCredentialError, 'not connect to'))

    @patch_cli(output='credential_error.txt')
    def test_credential_error_no_heart_beat(self):
        hb = NodeHeartBeat(interval=0.01)
        hb.add('spa', '1.1.1.1')
        hb.add('spb', '1.1.1.2')
        assert_that(hb.is_available('spa'), equal_to(True))
        assert_that(hb.is_available('spb'), equal_to(True))
        time.sleep(0.1)
        assert_that(hb.command_count, less_than_or_equal_to(2))
        assert_that(hb.command_count, less_than_or_equal_to(2))
        hb.stop()
        time.sleep(0.1)


class NodeInfoTest(TestCase):
    def test_repr(self):
        info = NodeInfo('spa', '1.1.1.1', True, False)
        info.latency = 5
        info.latency = 14
        expected = ('name: SP A, ip: 1.1.1.1, available: True, '
                    'working: False, latency: 10.0, timestamp: None')
        assert_that(repr(info), equal_to(expected))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, raises, instance_of

from storops_test.vnx.cli_mock import patch_cli, t_cli
from storops_test.vnx.resource.verifiers import verify_raid0, verify_raid_1
from storops.exception import VNXCreateRaidGroupError, \
    VNXDeleteRaidGroupError
from storops.vnx.resource.disk import VNXDisk
from storops.vnx.resource.rg import VNXRaidGroup

__author__ = 'Cedric Zhuang'


class VNXRaidGroupTest(TestCase):
    @patch_cli
    def test_get_rg(self):
        rg = VNXRaidGroup.get(t_cli(), 0)
        verify_raid0(rg)

    @patch_cli
    def test_get_rg_list(self):
        rgs = VNXRaidGroup.get(t_cli())
        assert_that(len(rgs), equal_to(7))
        for rg in rgs:
            if rg.raid_group_id == 0:
                verify_raid0(rg)
                break
        else:
            self.fail('RAID group 0 not found.')

    @patch_cli
    def test_get_unbound_rg(self):
        rg = VNXRaidGroup.get(t_cli(), 1)
        verify_raid_1(rg)

    @patch_cli
    def test_create_rg(self):
        def f():
            VNXRaidGroup.create(t_cli(), 11, ['1_0_0', '1_0_1'])

        assert_that(f, raises(VNXCreateRaidGroupError, 'not supported'))

    @patch_cli
    def test_create_rg_invalid_raid_type(self):
        def f():
            VNXRaidGroup.create(t_cli(), 11, ['1_0_0', '1_0_1'], 'r12')

        assert_that(f, raises(ValueError, 'valid value'))

    @patch_cli
    def test_delete_rg(self):
        def f():
            VNXRaidGroup(11, t_cli()).delete()

        assert_that(f, raises(VNXDeleteRaidGroupError, 'Not Found'))

    @patch_cli
    def test_disks(self):
        rg = VNXRaidGroup(4, t_cli())
        disks = rg.disks
        assert_that(len(disks), equal_to(5))
        for disk in disks:
            assert_that(disk, instance_of(VNXDisk))
            assert_that(disk.existed, equal_to(True))

    @patch_cli
    def test_available_capacity_gbs(self):
        rg = VNXRaidGroup(4, t_cli())
        available_capacity = rg.available_capacity_gbs
        assert_that(available_capacity, equal_to(0.943))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, has_items, close_to, \
    contains_string, instance_of, has_length, none

from storops import DiskTechnologyEnum, TierTypeEnum, HotSparePolicyStatusEnum
from storops.unity.resource.disk import UnityDiskList, UnityDisk, \
    UnityDiskGroup, UnityDiskGroupList
from storops_test.unity.rest_mock import t_rest, patch_rest, t_unity

__author__ = 'Cedric Zhuang'


class UnityDiskTest(TestCase):
    @patch_rest
    def test_get_all(self):
        disks = UnityDiskList(cli=t_rest())
        assert_that(len(disks), equal_to(40))
        self.verify_dae_0_1_disk_0(
            *filter(lambda d: d.id == 'dae_0_1_disk_0', disks))

    @patch_rest
    def test_properties(self):
        disk = UnityDisk(_id='dae_0_1_disk_0', cli=t_rest())
        self.verify_dae_0_1_disk_0(disk)

    @patch_rest
    def test_nested_properties(self):
        disk = UnityDisk(_id='dpe_disk_12', cli=t_rest())
        assert_that(disk.pool.id, equal_to('pool_1'))
        assert_that(disk.pool.name, equal_to('perfpool1130'))

    def verify_dae_0_1_disk_0(self, disk):
        assert_that(disk.id, equal_to('dae_0_1_disk_0'))
        assert_that(disk.bus_id, equal_to(0))
        assert_that(disk.current_speed, equal_to(12000000000))
        assert_that(disk.disk_technology, equal_to(DiskTechnologyEnum.NL_SAS))
        assert_that(disk.emc_part_number, equal_to('005051284'))
        assert_that(disk.emc_serial_number, equal_to('Z4H027TW'))
        assert_that(disk.is_fast_cache_in_use, equal_to(False))
        assert_that(disk.is_in_use, equal_to(False))
        assert_that(disk.is_sed, equal_to(False))
        assert_that(disk.manufacturer, equal_to('SEAGATE'))
        assert_that(disk.max_speed, equal_to(12000000000))
        assert_that(disk.model, equal_to('ST2000NK EMC2000'))
        assert_that(disk.name, equal_to('DAE 0 1 Disk 0'))
        assert_that(disk.needs_replacement, equal_to(False))
        assert_that(disk.raw_size, equal_to(1969623564288))
        assert_that(disk.rpm, equal_to(7200))
        assert_that(disk.size, equal_to(1969590009856))
        assert_that(disk.slot_number, equal_to(0))
        assert_that(disk.tier_type, equal_to(TierTypeEnum.CAPACITY))
        assert_that(disk.vendor_size, equal_to(2199023255552))
        assert_that(disk.version, equal_to('MN16'))
        assert_that(disk.wwn, equal_to(
            '06:00:00:00:05:00:00:00:0C:01:00:00:00:00:00:03'))
        assert_that(disk.inserted, equal_to(True))

    def test_metric_names(self):
        metric_names = UnityDisk().metric_names()
        assert_that(metric_names, has_items('read_iops', 'write_iops'))

    @patch_rest
    def test_not_inserted(self):
        disk = UnityDisk(_id='dpe_disk_23', cli=t_rest())
        assert_that(disk.inserted, equal_to(False))

    @patch_rest
    def test_metric_read_iops(self):
        unity = t_unity()
        disk = UnityDisk(_id='dae_0_1_disk_2', cli=unity._cli)
        assert_that(disk.write_iops, close_to(0.0, 0.01))

    @patch_rest
    def test_metric_in_repr(self):
        unity = t_unity()
        disk = UnityDisk(_id='dae_0_1_disk_2', cli=unity._cli)
        repr_str = repr(disk)
        assert_that(repr_str, contains_string('"read_iops":'))
        assert_that(repr_str, contains_string('"write_mbps"'))

    @patch_rest
    def test_disk_list_metrics(self):
        unity = t_unity()
        disks = UnityDiskList(cli=unity._cli)
        assert_that(len(disks), equal_to(40))

    @patch_rest
    def test_disk_utilization(self):
        unity = t_unity()
        disk = UnityDisk(_id='dae_0_1_disk_2', cli=unity._cli)
        assert_that(disk.utilization, close_to(2.41, 0.01))

    @patch_rest
    def test_get_disk_inserted_list(self):
        disks = UnityDiskList(cli=t_rest(), inserted=True)
        ret = set(disks.inserted)
        assert_that(ret, equal_to({True}))
        assert_that(len(disks), equal_to(26))


class DiskGroupTest(TestCase):

    @patch_rest
    def test_get_all(self):
        t_cli = t_rest()
        disk_groups = UnityDiskGroupList.get(cli=t_cli)
        assert_that(disk_groups, instance_of(UnityDiskGroupList))
        assert_that(disk_groups, has_length(4))

    @patch_rest
    def test_get_one(self):
        disk_group = UnityDiskGroup.get(cli=t_rest(), _id='dg_15')
        assert_that(disk_group, instance_of(UnityDiskGroup))
        assert_that(disk_group.id, equal_to('dg_15'))
        assert_that(disk_group.disk_technology,
                    equal_to(DiskTechnologyEnum.SAS))
        assert_that(disk_group.name, equal_to('600 GB SAS 15K'))
        assert_that(disk_group.is_fast_cache_allowable, equal_to(False))
        assert_that(disk_group.disk_size, equal_to(590894538752))
        assert_that(disk_group.advertised_size, equal_to(644245094400))
        assert_that(disk_group.rpm, equal_to(15000))
        assert_that(disk_group.speed, equal_to(9))
        assert_that(disk_group.total_disks, equal_to(12))
        assert_that(disk_group.unconfigured_disks, equal_to(8))
        assert_that(disk_group.min_hot_spare_candidates, equal_to(1))
        assert_that(disk_group.hot_spare_policy_status,
                    HotSparePolicyStatusEnum.OK)
        assert_that(disk_group.configured_traditional_pool_disks, equal_to(4))
        assert_that(disk_group.configured_fast_cache_disks, equal_to(0))
        assert_that(disk_group.disks_past_eol, none())
        assert_that(disk_group.disks_with_eol_less30_days, none())
        assert_that(disk_group.disks_with_eol_less60_days, none())
        assert_that(disk_group.disks_with_eol_less90_days, none())
        assert_that(disk_group.disks_with_eol_less180_days, none())



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import logging

from storops.vnx import xmlapi
from storops.vnx.resource.fs import VNXFileSystem
from storops.vnx.resource.mover import VNXMover
from storops.vnx.resource import VNXResource, VNXCliResourceList

__author__ = 'Jay Xu'

log = logging.getLogger(__name__)


class NfsHostConfig(object):
    def __init__(self, root_hosts=None, ro_hosts=None, rw_hosts=None,
                 access_hosts=None):
        self.root_hosts = root_hosts
        self.ro_hosts = ro_hosts
        self.rw_hosts = rw_hosts
        self.access_hosts = access_hosts

    def get_xml_node(self):
        xb = xmlapi.XmlBuilder()

        ret = []
        if self.access_hosts is not None:
            ret.append(xb.list_elements('AccessHosts', self.access_hosts))
        if self.rw_hosts is not None:
            ret.append(xb.list_elements('RwHosts', self.rw_hosts))
        if self.ro_hosts is not None:
            ret.append(xb.list_elements('RoHosts', self.ro_hosts))
        if self.root_hosts is not None:
            ret.append(xb.list_elements('RootHosts', self.root_hosts))
        return ret

    @staticmethod
    def _add(left, right):
        if left is None:
            left = []
        return list(set(left + list(right)))

    @staticmethod
    def _delete(left, right):
        if left is None:
            ret = None
        else:
            ret = set(left) - set(right)
        return ret

    def add_access_and_root_hosts(self, *hosts):
        self.root_hosts = self._add(self.root_hosts, hosts)
        self.access_hosts = self._add(self.access_hosts, hosts)

    def add_ro_hosts(self, *hosts):
        self.ro_hosts = self._add(self.ro_hosts, hosts)
        self.add_access_and_root_hosts(*hosts)

    def add_rw_hosts(self, *hosts):
        self.rw_hosts = self._add(self.rw_hosts, hosts)
        self.add_access_and_root_hosts(*hosts)

    def delete_hosts(self, *hosts):
        self.rw_hosts = self._delete(self.rw_hosts, hosts)
        self.ro_hosts = self._delete(self.ro_hosts, hosts)
        self.access_hosts = self._delete(self.access_hosts, hosts)
        self.root_hosts = self._delete(self.root_hosts, hosts)


class VNXNfsShareList(VNXCliResourceList):
    def __init__(self, cli=None, mover=None, path=None):
        super(VNXNfsShareList, self).__init__(cli)
        self._mover = mover
        self._path = path

    @classmethod
    def get_resource_class(cls):
        return VNXNfsShare

    def _get_raw_resource(self):
        if self._mover is not None:
            mover_id = VNXMover.get_id(self._mover)
        else:
            mover_id = None
        return self._cli.get_nfs_export(mover_id, self._path)


class VNXNfsShare(VNXResource):
    def __init__(self, mover=None, path=None, cli=None):
        super(VNXNfsShare, self).__init__()
        self._cli = cli
        self._mover = mover
        self._path = path

    def _get_raw_resource(self):
        if self._mover is not None:
            mover_id = VNXMover.get_id(self._mover)
        else:
            raise ValueError('mover for the nfs share is not specified.')
        if self._path is None:
            raise ValueError('path for the nfs share is not specified')
        return self._cli.get_nfs_export(mover_id, self._path)

    @staticmethod
    def get(cli, mover=None, path=None):
        if mover is not None and path is not None:
            ret = VNXNfsShare(mover=mover, path=path, cli=cli)
        else:
            ret = VNXNfsShareList(cli=cli, mover=mover, path=path)
        return ret

    @property
    def mover(self):
        if self._mover is not None:
            ret = self._mover
        else:
            ret = VNXMover(mover_id=self.mover_id, cli=self._cli)
        return ret

    @property
    def fs(self):
        return VNXFileSystem(fs_id=self.fs_id, cli=self._cli)

    def get_mover_id(self):
        return VNXMover.get_id(self._mover)

    @staticmethod
    def create(cli, mover, path, ro=False, host_config=None):
        mover_id = VNXMover.get_id(mover)
        resp = cli.create_nfs_export(mover_id, path, ro,
                                     host_config=host_config)
        resp.raise_if_err()
        return VNXNfsShare(cli=cli, mover=mover, path=path)

    def get_path(self):
        if self._path is not None:
            ret = self._path
        else:
            ret = self.path
        return ret

    def delete(self):
        mover_id = self.get_mover_id()
        resp = self._cli.delete_nfs_export(mover_id, self.get_path())
        resp.raise_if_err()
        return resp

    def modify(self, ro=None, host_config=None):
        mover_id = self.get_mover_id()
        path = self.get_path()
        resp = self._cli.modify_nfs_export(mover_id, path, ro, host_config)
        resp.raise_if_err()
        return resp

    @property
    def host_config(self):
        return NfsHostConfig(root_hosts=self.root_hosts,
                             ro_hosts=self.ro_hosts,
                             rw_hosts=self.rw_hosts,
                             access_hosts=self.access_hosts)

    def allow_ro_access(self, *hosts):
        host_config = self.host_config
        host_config.add_ro_hosts(*hosts)
        resp = self.modify(host_config=host_config)
        resp.raise_if_err()
        return resp

    def allow_rw_access(self, *hosts):
        host_config = self.host_config
        host_config.add_rw_hosts(*hosts)
        resp = self.modify(host_config=host_config)
        resp.raise_if_err()
        return resp

    def deny_access(self, *hosts):
        host_config = self.host_config
        host_config.delete_hosts(*hosts)
        resp = self.modify(host_config=host_config)
        resp.raise_if_err()
        return resp



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, instance_of, contains_string, \
    has_item

from storops.exception import UnityFileSystemSizeTooSmallError
from storops.unity.enums import JobStateEnum, JobTaskStateEnum
from storops.unity.resource.job import UnityJob, UnityJobList, \
    UnityJobTaskList, \
    UnityMessageList, UnityLocalizedMessageList
from storops.unity.resource.pool import UnityPool
from storops.unity.resource.nas_server import UnityNasServer
from storops import exception as ex

from storops_test.unity.rest_mock import t_rest, patch_rest

__author__ = 'Cedric Zhuang'


class UnityJobTest(TestCase):
    @staticmethod
    def test_job():
        return UnityJob(_id='N-345', cli=t_rest())

    @patch_rest
    def test_job_properties(self):
        job = self.test_job()
        assert_that(job.id, equal_to('N-345'))
        assert_that(job.state, equal_to(JobStateEnum.COMPLETED))
        assert_that(job.description, equal_to('Delete storage resource'))
        end_time = '2016-03-22 10:39:53.561000+00:00'
        assert_that(str(job.state_change_time), equal_to(end_time))
        assert_that(str(job.end_time), equal_to(end_time))
        assert_that(str(job.submit_time),
                    equal_to('2016-03-22 10:39:20.033000+00:00'))
        assert_that(str(job.start_time),
                    equal_to('2016-03-22 10:39:20.184000+00:00'))
        assert_that(job.progress_pct, equal_to(100))
        assert_that(str(job.elapsed_time), equal_to('0:00:33.377000'))

    @patch_rest
    def test_task_properties(self):
        job = self.test_job()
        tasks = job.tasks
        assert_that(tasks, instance_of(UnityJobTaskList))
        assert_that(len(tasks), equal_to(4))

        task = tasks[0]
        assert_that(task.state, equal_to(JobTaskStateEnum.COMPLETED))
        assert_that(task.name, equal_to(
            'job.applicationprovisioningservice.task.'
            'DeleteApplicationPrecondition613'))
        assert_that(task.description,
                    equal_to('Check storage resource state before deletion'))
        assert_that(task.parameters_in, instance_of(dict))
        assert_that(task.parameters_in['deleteRemotePeer'], equal_to(True))
        assert_that(task.parameters_in['id'], equal_to('RS_1'))
        assert_that(task.parameters_out, instance_of(dict))
        assert_that(task.parameters_out['id'], equal_to('B-2'))

    @patch_rest
    def test_message_properties(self):
        job = self.test_job()
        messages = job.tasks[0].messages
        assert_that(messages, instance_of(UnityMessageList))
        assert_that(len(messages), equal_to(2))

        message = messages[0]
        assert_that(message.error_code, equal_to(0))

        localized_messages = message.messages
        assert_that(localized_messages, instance_of(UnityLocalizedMessageList))

        localized_message = localized_messages[0]
        assert_that(localized_message.locale, equal_to('en_US'))
        assert_that(localized_message.message, equal_to('Success'))

    @patch_rest
    def test_get_all(self):
        jobs = UnityJobList(cli=t_rest())
        assert_that(len(jobs), equal_to(3))

    @patch_rest
    def test_create_nfs_share_async(self):
        pool = UnityPool.get(cli=t_rest(), _id='pool_5')
        nas_server = UnityNasServer.get(cli=t_rest(), _id='nas_6')
        job = UnityJob.create_nfs_share(
            cli=t_rest(), pool=pool, nas_server=nas_server,
            name='513dd8b0-2c22-4da0-888e-494d320303b6',
            size=4294967296)
        assert_that(JobStateEnum.COMPLETED, equal_to(job.state))

    @patch_rest
    def test_create_nfs_share_failed(self):
        pool = UnityPool.get(cli=t_rest(), _id='pool_5')
        nas_server = UnityNasServer.get(cli=t_rest(), _id='nas_6')
        self.assertRaisesRegexp(
            ex.JobStateError,
            'Job State: FAILED.  Error Detail: ',
            UnityJob.create_nfs_share,
            cli=t_rest(), pool=pool, nas_server=nas_server,
            name='613dd8b0-2c22-4da0-888e-494d320303b7',
            size=4294967296,
            async_mode=False)

    @patch_rest
    def test_messages(self):
        job = UnityJob(_id='B-3', cli=t_rest())
        assert_that(len(job.messages), equal_to(1))
        assert_that(job.messages[0], contains_string('too small'))

    @patch_rest
    def test_has_exceptions(self):
        job = UnityJob(_id='B-3', cli=t_rest())
        assert_that(len(job.exceptions), equal_to(1))
        exception = job.exceptions[0]
        assert_that(exception,
                    instance_of(UnityFileSystemSizeTooSmallError))
        assert_that(str(exception), contains_string('too small'))
        assert_that(job.has_exception(), equal_to(True))

    @patch_rest
    def test_batch_without_error(self):
        job = UnityJob(_id='B-693', cli=t_rest())
        assert_that(job.messages, has_item('Success'))
        assert_that(len(job.exceptions), equal_to(0))
        assert_that(job.has_exception(), equal_to(False))

    @patch_rest
    def test_normal_without_error(self):
        job = UnityJob(_id='N-345', cli=t_rest())
        assert_that(job.messages, has_item('Success'))
        assert_that(len(job.exceptions), equal_to(0))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, only_contains, instance_of, raises

from storops.exception import UnityOneDnsPerNasServerError, \
    UnityResourceNotFoundError
from storops.unity.resource.dns_server import UnityFileDnsServerList, \
    UnityFileDnsServer
from storops.unity.resource.nas_server import UnityNasServer
from storops_test.unity.rest_mock import t_rest, patch_rest

__author__ = 'Cedric Zhuang'


class UnityFileDnsServerTest(TestCase):
    @patch_rest
    def test_get_properties(self):
        server = UnityFileDnsServer('dns_2', cli=t_rest())
        assert_that(server.existed, equal_to(True))
        assert_that(server.addresses, only_contains('10.244.209.72'))
        assert_that(server.domain, equal_to('win2012.dev'))
        assert_that(server.nas_server, instance_of(UnityNasServer))

    @patch_rest
    def test_get_all(self):
        servers = UnityFileDnsServerList(cli=t_rest())
        assert_that(len(servers), equal_to(1))

    @patch_rest
    def test_create_one_dns_each_nas_server(self):
        def f():
            UnityFileDnsServer.create(t_rest(), 'nas_2', 'emc.dev',
                                      ['2.2.2.2', '3.3.3.3'])

        assert_that(f, raises(UnityOneDnsPerNasServerError, 'Only one DNS'))

    @patch_rest
    def test_create_success(self):
        server = UnityNasServer.get(t_rest(), 'nas_4')
        dns = UnityFileDnsServer.create(t_rest(), server, 'emc.dev',
                                        ['2.2.2.2', '3.3.3.3'])
        assert_that(dns.addresses, only_contains('2.2.2.2', '3.3.3.3'))

    @patch_rest
    def test_delete_not_found(self):
        def f():
            UnityFileDnsServer.get(t_rest(), 'dns_30').delete()

        assert_that(f, raises(UnityResourceNotFoundError, 'does not exist'))



# coding=utf-8
# Copyright (c) 2016 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, only_contains

from storops.unity.parser import NestedProperties


class NestedPropertiesTest(TestCase):

    def test_build_nested_properties(self):
        np1 = NestedProperties.build(None)
        assert_that(np1, equal_to(None))
        np2 = NestedProperties.build('aa.bb')
        assert_that(np2.query_fields, only_contains('aa.bb'))
        np3 = NestedProperties.build(('aa_bb.cc', 'cc.dd'))
        assert_that(np3.query_fields, only_contains('aaBb.cc', 'cc.dd'))

    def test_get_properties(self):
        nested_props = NestedProperties('ab_cd.ef.c',
                                        'aaa_bb.ccc')
        properties = nested_props.get_properties()
        assert_that(properties, only_contains('ab_cd', 'aaa_bb'))

    def test_query_fields(self):
        nested_props = NestedProperties('ab_cd.ef.c',
                                        'aaa_bb.ccc_dd')
        assert_that(nested_props.query_fields, only_contains('abCd.ef.c',
                                                             'aaaBb.cccDd'))

    def test_get_child_subtree(self):
        nested_props = NestedProperties('ab_cd.ef.c',
                                        'ab_cd.ef.d',
                                        'aaa_bb.ccc_dd',
                                        'aaa_bb.ee_ff')
        sub1 = nested_props.get_child_subtree('ab_cd')
        assert_that(sub1.get_properties(), only_contains('ef'))
        sub1sub = sub1.get_child_subtree('ef')
        assert_that(sub1sub.get_properties(), only_contains('c', 'd'))
        sub2 = nested_props.get_child_subtree('aaa_bb')
        assert_that(sub2.get_properties(), only_contains('ccc_dd', 'ee_ff'))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import storops.vnx.resource.lun
from storops.lib.common import instance_cache
from storops.vnx.resource import VNXCliResourceList, VNXCliResource

__author__ = 'Cedric Zhuang'


class VNXMigrationSessionList(VNXCliResourceList):
    @classmethod
    def get_resource_class(cls):
        return VNXMigrationSession

    def _get_raw_resource(self):
        return self._cli.get_migration_session(poll=self.poll)


class VNXMigrationSession(VNXCliResource):
    def __init__(self, source=None, cli=None):
        super(VNXMigrationSession, self).__init__()
        self._cli = cli
        self._source = source

    def _get_raw_resource(self):
        source_id = storops.vnx.resource.lun.VNXLun.get_id(self._source)
        return self._cli.get_migration_session(source_id, poll=self.poll)

    @property
    def is_migrating(self):
        return self.current_state == 'MIGRATING'

    @property
    def is_success(self):
        return self.current_state in ('MIGRATED', None)

    @classmethod
    def get(cls, cli, source=None):
        if source is None:
            ret = VNXMigrationSessionList(cli)
        else:
            ret = VNXMigrationSession(source, cli)
        return ret

    @property
    @instance_cache
    def source_lun(self):
        if self._source is not None:
            return storops.vnx.resource.lun.VNXLun(
                lun_id=self._source,
                cli=self._cli)
        return storops.vnx.resource.lun.VNXLun.get(
            cli=self._cli, lun_id=self.source_lu_id, name=self.source_lu_name)

    @property
    @instance_cache
    def destination_lun(self):
        return storops.vnx.resource.lun.VNXLun.get(
            cli=self._cli, lun_id=self.dest_lu_id, name=self.dest_lu_name)

    def cancel(self):
        self.source_lun.cancel_migrate()



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import logging

from storops_comptest.utils import inter_process_locked
from storops import VNXSystem, UnitySystem, cache

__author__ = 'Cedric Zhuang'

log = logging.getLogger(__name__)


@inter_process_locked('t_vnx.lck')
@cache
def t_vnx():
    vnx = VNXSystem('10.244.211.30', 'sysadmin', 'sysadmin')
    log.debug('initialize vnx system: {}'.format(vnx))
    return vnx


@inter_process_locked('t_unity.lck')
@cache
def t_unity():
    unity = UnitySystem('10.244.223.61', 'admin', 'Password123!')
    log.debug('initialize unity system: {}'.format(unity))
    return unity


@inter_process_locked('t_vnx.lck')
@cache
def vnx1():
    vnx = VNXSystem('192.168.1.52', 'sysadmin', 'sysadmin')
    log.debug('initialize vnx system: {}'.format(vnx))
    return vnx


@inter_process_locked('t_vnx.lck')
@cache
def vnx2():
    vnx = VNXSystem('192.168.1.94', 'sysadmin', 'sysadmin')
    log.debug('initialize vnx system: {}'.format(vnx))
    return vnx



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import unittest

from hamcrest import assert_that, greater_than_or_equal_to, equal_to, raises

from storops_test.vnx.nas_mock import t_nas, patch_post
from storops.exception import VNXBackendError
from storops.vnx.resource.mount_point import VNXFsMountPointList, \
    VNXFsMountPoint
from storops.vnx.resource.mover import VNXMover

__author__ = 'Jay Xu'


class VNXFsMountPointTest(unittest.TestCase):
    @patch_post
    def test_get_all(self):
        mps = VNXFsMountPointList(cli=t_nas())
        assert_that(len(mps), greater_than_or_equal_to(1))
        mp = next(mp for mp in mps if mp.path == '/zhuanc_fs_100g')
        self.verify_fs_100g(mp)

    @patch_post
    def test_get_all_by_mover(self):
        mover = VNXMover(mover_id=2, cli=t_nas())
        mps = VNXFsMountPointList(mover=mover, cli=t_nas())
        assert_that(len(mps), greater_than_or_equal_to(1))

    @patch_post
    def test_get_by_path(self):
        mover = VNXMover(mover_id=1)
        mp = VNXFsMountPoint(mover=mover, path='/zhuanc_fs_100g', cli=t_nas())
        self.verify_fs_100g(mp)

    @patch_post
    def test_get_not_found(self):
        mover = VNXMover(mover_id=1)
        mp = VNXFsMountPoint(mover=mover, path='/not_found', cli=t_nas())
        assert_that(mp.existed, equal_to(False))

    @staticmethod
    def verify_fs_100g(mp):
        assert_that(mp.mover_id, equal_to(1))
        assert_that(mp.existed, equal_to(True))
        assert_that(mp.is_vdm, equal_to(False))
        assert_that(mp.disabled, equal_to(False))
        assert_that(mp.path, equal_to('/zhuanc_fs_100g'))
        assert_that(mp.fs_id, equal_to(211))
        assert_that(mp.nt_credential, equal_to(False))

    @patch_post
    def test_create_fs_mp_path_occupied(self):
        def f():
            cli = t_nas()
            mover = VNXMover(mover_id=1, cli=cli)
            VNXFsMountPoint.create(cli, '/zhuanc_fs_100g', 244, mover)

        assert_that(f, raises(VNXBackendError, 'currently mounted'))

    @patch_post
    def test_create_fs_mp_invalid_fs(self):
        def f():
            cli = t_nas()
            mover = VNXMover(mover_id=1, cli=cli)
            VNXFsMountPoint.create(cli, '/zhuanc_fs_100g', 24, mover)

        assert_that(f, raises(VNXBackendError, 'invalid filesystem specified'))

    @patch_post
    def test_create_fs_mp_mounted(self):
        def f():
            cli = t_nas()
            mover = VNXMover(mover_id=2, cli=cli)
            VNXFsMountPoint.create(cli, '/zhuanc_fs_100g', 244, mover)

        assert_that(f, raises(VNXBackendError, 'is mounted on'))

    @patch_post
    def test_create_fs_mp_success(self):
        cli = t_nas()
        mover = VNXMover(mover_id=2, cli=cli)
        VNXFsMountPoint.create(cli, '/fs_100g', 244, mover)

    @patch_post
    def test_delete_fs_mp(self):
        cli = t_nas()
        mover = VNXMover(mover_id=2, cli=cli)
        resp = VNXFsMountPoint(mover, '/testfs', cli).delete()
        assert_that(resp.is_ok(), equal_to(True))

    @patch_post
    def test_same_path_different_mover(self):
        mps = VNXFsMountPointList(cli=t_nas(), path='/same')
        assert_that(len(mps), equal_to(2))
        for mp in mps:
            assert_that(mp.path, equal_to('/same'))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, instance_of, raises
from storops.unity.resource.snap_schedule import UnitySnapSchedule

from storops.exception import UnityLunNameInUseError, JobStateError, \
    UnityPoolNameInUseError
from storops.unity.enums import RaidTypeEnum, FastVPStatusEnum, \
    FastVPRelocationRateEnum, PoolDataRelocationTypeEnum, \
    RaidStripeWidthEnum, TierTypeEnum, PoolUnitTypeEnum, \
    FSSupportedProtocolEnum, TieringPolicyEnum, JobStateEnum, \
    StoragePoolTypeEnum, ESXFilesystemMajorVersionEnum, \
    ESXFilesystemBlockSizeEnum
from storops.unity.resource.disk import UnityDiskGroup, UnityDisk
from storops.unity.resource.pool import UnityPool, UnityPoolList, \
    RaidGroupParameter
from storops.unity.resource.lun import UnityLun
from storops.unity.resource.sp import UnityStorageProcessor
from storops.unity.resource.nas_server import UnityNasServer

from storops_test.unity.rest_mock import t_rest, patch_rest

__author__ = 'Cedric Zhuang'


class UnityPoolTest(TestCase):
    @patch_rest
    def test_properties(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        self.verify_pool_1(pool)

    @staticmethod
    def verify_pool_1(pool):
        assert_that(pool.id, equal_to('pool_1'))
        assert_that(pool.raid_type, equal_to(RaidTypeEnum.MIXED))
        assert_that(pool.name, equal_to('perfpool1130'))
        assert_that(pool.description, equal_to('pp'))
        assert_that(pool.size_free, equal_to(9160359936000))
        assert_that(pool.size_total, equal_to(9251627991040))
        assert_that(pool.size_used, equal_to(91268055040))
        assert_that(pool.size_subscribed, equal_to(1392106274816))
        assert_that(pool.alert_threshold, equal_to(70))
        assert_that(pool.pool_space_harvest_high_threshold, equal_to(95.0))
        assert_that(pool.pool_space_harvest_low_threshold, equal_to(85.0))
        assert_that(pool.snap_space_harvest_high_threshold, equal_to(25.0))
        assert_that(pool.snap_space_harvest_low_threshold, equal_to(20.0))
        assert_that(pool.is_fast_cache_enabled, equal_to(False))
        assert_that(str(pool.creation_time),
                    equal_to('2016-02-29 07:34:23+00:00'))
        assert_that(pool.is_empty, equal_to(False))
        assert_that(pool.is_harvest_enabled, equal_to(True))
        assert_that(pool.is_snap_harvest_enabled, equal_to(False))
        assert_that(pool.metadata_size_subscribed, equal_to(59324235776))
        assert_that(pool.snap_size_subscribed, equal_to(873220538368))
        assert_that(pool.metadata_size_used, equal_to(36775657472))
        assert_that(pool.snap_size_used, equal_to(24452407296))
        assert_that(pool.is_all_flash, equal_to(False))
        assert_that(pool.pool_type, equal_to(StoragePoolTypeEnum.TRADITIONAL))
        tiers = pool.tiers

        assert_that(len(tiers), equal_to(3))
        for tier in tiers:
            assert_that(tier._cli, equal_to(pool._cli))

        assert_that(pool.pool_fast_vp._cli, equal_to(pool._cli))

    @patch_rest
    def test_pool_fast_vp_properties(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        fast = pool.pool_fast_vp
        assert_that(fast.status, equal_to(FastVPStatusEnum.ACTIVE))
        assert_that(fast.relocation_rate,
                    equal_to(FastVPRelocationRateEnum.MEDIUM))
        assert_that(fast.type, equal_to(PoolDataRelocationTypeEnum.SCHEDULED))
        assert_that(fast.is_schedule_enabled, equal_to(True))
        assert_that(str(fast.relocation_duration_estimate),
                    equal_to('0:00:00'))
        assert_that(fast.size_moving_down, equal_to(0))
        assert_that(fast.size_moving_up, equal_to(0))
        assert_that(fast.size_moving_within, equal_to(0))
        assert_that(fast.percent_complete, equal_to(0))
        assert_that(fast.data_relocated, equal_to(0))
        assert_that(str(fast.last_start_time),
                    equal_to('2016-03-13 22:00:00+00:00'))
        assert_that(str(fast.last_end_time),
                    equal_to('2016-03-14 06:00:00+00:00'))

    @patch_rest
    def test_tier_properties(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        tier = next(t for t in pool.tiers if t.name == 'Performance')
        assert_that(tier.raid_type, equal_to(RaidTypeEnum.RAID5))
        assert_that(tier.stripe_width, equal_to(RaidStripeWidthEnum._5))
        assert_that(tier.tier_type, equal_to(TierTypeEnum.PERFORMANCE))
        assert_that(tier.size_total, equal_to(1180847570944))
        assert_that(tier.size_used, equal_to(3489660928))
        assert_that(tier.size_free, equal_to(1177357910016))
        assert_that(tier.size_moving_down, equal_to(0))
        assert_that(tier.size_moving_up, equal_to(0))
        assert_that(tier.size_moving_within, equal_to(0))
        assert_that(tier.disk_count, equal_to(5))

    @patch_rest
    def test_get_all(self):
        pools = UnityPoolList(cli=t_rest())
        assert_that(len(pools), equal_to(2))

        pool = next(pool for pool in pools if pool.id == 'pool_1')
        self.verify_pool_1(pool)

    @patch_rest
    def test_disk_groups(self):
        cli = t_rest()
        pool0 = UnityPool.get(cli=cli, _id='pool_1')
        disk_groups = pool0.disk_groups
        assert_that(disk_groups, instance_of(dict))
        assert_that(len(disk_groups), equal_to(2))
        assert_that(disk_groups['dg_8'], instance_of(list))
        assert_that(disk_groups['dg_15'], instance_of(list))
        for key in disk_groups:
            for disk in disk_groups[key]:
                assert_that(disk, instance_of(UnityDisk))

    @patch_rest
    def test_get_nested_resource_properties(self):
        pools = UnityPoolList(cli=t_rest())
        pool = next(pool for pool in pools if pool.id == 'pool_1')
        tier = next(t for t in pool.tiers if t.name == 'Performance')
        unit = next(u for u in tier.pool_units if u.id == 'rg_2')
        assert_that(unit.type, equal_to(PoolUnitTypeEnum.RAID_GROUP))
        assert_that(unit.tier_type, equal_to(TierTypeEnum.PERFORMANCE))
        assert_that(unit.name, equal_to("RAID5, #2, pool:perfpool1130"))
        assert_that(unit.description, equal_to('123'))
        assert_that(unit.wwn, equal_to(
            '06:00:00:00:05:00:00:00:01:00:00:00:00:00:00:64'))
        assert_that(unit.size_total, equal_to(1181501882368))
        assert_that(unit.pool, instance_of(UnityPool))

    @patch_rest
    def test_get_nested_resource_filter_by_non_id(self):
        pools = UnityPoolList(cli=t_rest())
        pool = next(pool for pool in pools if pool.id == 'pool_1')
        tier = next(t for t in pool.tiers if t.name == 'Performance')
        unit = next(u for u in tier.pool_units if u.description == '123')
        assert_that(unit.id, equal_to('rg_2'))

    @patch_rest
    def test_create_pool(self):
        cli = t_rest()
        disk_group = UnityDiskGroup.get(cli=cli, _id='dg_15')
        raid_group_0 = RaidGroupParameter(
            disk_group=disk_group,
            disk_num=3, raid_type=RaidTypeEnum.RAID5,
            stripe_width=RaidStripeWidthEnum.BEST_FIT)
        raid_groups = [raid_group_0]
        pool = UnityPool.create(
            cli=cli, name='test_pool', description='Unity test pool.',
            raid_groups=raid_groups, alert_threshold=15,
            is_harvest_enabled=True, is_snap_harvest_enabled=True,
            pool_harvest_high_threshold=80, pool_harvest_low_threshold=40,
            snap_harvest_high_threshold=80, snap_harvest_low_threshold=40,
            is_fast_cache_enabled=True, is_fastvp_enabled=True,
            pool_type=StoragePoolTypeEnum.DYNAMIC)
        assert_that(pool.id, equal_to('pool_4'))
        assert_that(pool.pool_type, equal_to(StoragePoolTypeEnum.DYNAMIC))
        assert_that(pool.is_all_flash, equal_to(False))

    @patch_rest
    def test_create_pool_name_in_use(self):
        cli = t_rest()
        raid_group_0 = RaidGroupParameter(
            disk_group='dg_15',
            disk_num=3, raid_type=RaidTypeEnum.RAID5,
            stripe_width=RaidStripeWidthEnum.BEST_FIT)
        raid_groups = [raid_group_0]

        def _inner():
            UnityPool.create(
                cli=cli, name='duplicate_pool',
                description='Unity test pool.',
                raid_groups=raid_groups)

        assert_that(_inner, raises(UnityPoolNameInUseError))

    @patch_rest
    def test_extend_pool(self):
        cli = t_rest()
        raid_group_0 = RaidGroupParameter(
            disk_group='dg_8',
            disk_num=4,
            raid_type=RaidTypeEnum.RAID10,
            stripe_width=RaidStripeWidthEnum.BEST_FIT)
        raid_groups = [raid_group_0]
        pool0 = UnityPool.get(cli=cli, _id='pool_1')
        resp = pool0.modify(raid_groups=raid_groups)
        assert_that(resp.is_ok(), equal_to(True))

    @patch_rest
    def test_modify_pool(self):
        cli = t_rest()
        pool0 = UnityPool.get(cli=cli, _id='pool_30')
        resp = pool0.modify(name="new_name",
                            is_fastvp_enabled=True,
                            alert_threshold=80)
        assert_that(resp.is_ok(), equal_to(True))

    @patch_rest
    def test_delete_pool(self):
        cli = t_rest()
        pool = UnityPool.get(cli=cli, _id='pool_4')
        resp = pool.delete()
        assert_that(resp.is_ok(), equal_to(True))

    @patch_rest
    def test_create_filesystem_success(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        fs = pool.create_filesystem(
            'nas_2', 'fs3', 3 * 1024 ** 3,
            proto=FSSupportedProtocolEnum.CIFS,
            tiering_policy=TieringPolicyEnum.AUTOTIER_HIGH)
        assert_that(fs.get_id(), equal_to('fs_12'))

    @patch_rest
    def test_create_lun(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        lun = pool.create_lun("LunName", 100)
        assert_that(lun, instance_of(UnityLun))

    @patch_rest
    def test_create_lun_with_same_name(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())

        def f():
            pool.create_lun("openstack_lun")

        assert_that(f, raises(UnityLunNameInUseError))

    @patch_rest
    def test_create_lun_on_spb(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        sp = UnityStorageProcessor(_id='spb', cli=t_rest())
        lun = pool.create_lun("LunName", 100, sp=sp)
        assert_that(lun, instance_of(UnityLun))

    @patch_rest
    def test_create_lun_with_muitl_property(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        lun = pool.create_lun("LunName", 100,
                              description="Hello World", is_thin=True,
                              is_repl_dst=True,
                              tiering_policy=TieringPolicyEnum.AUTOTIER_HIGH)
        assert_that(lun, instance_of(UnityLun))

    @patch_rest
    def test_create_lun_with_compression_enabled_v4_2(self):
        pool = UnityPool(_id='pool_1', cli=t_rest(version='4.2'))
        lun = pool.create_lun("LunName", 100,
                              is_compression=True)
        assert_that(lun, instance_of(UnityLun))

    @patch_rest
    def test_create_lun_with_compression_enabled(self):
        pool = UnityPool(_id='pool_1', cli=t_rest(version='4.3'))
        lun = pool.create_lun("LunName", 100,
                              is_compression=True)
        assert_that(lun, instance_of(UnityLun))

    @patch_rest
    def test_create_vmfs(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        vmfs = pool.create_vmfs(vmfs_name="VMFS datastore", size_gb=100)
        assert_that(vmfs, instance_of(UnityLun))

    @patch_rest
    def test_create_vmfs_vmware_iscsi_paramters(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        vmfs = pool.create_vmfs(
            vmfs_name="VMFS datastore 2", size_gb=100,
            major_version=ESXFilesystemMajorVersionEnum.VMFS_5,
            block_size=ESXFilesystemBlockSizeEnum._2MB)
        assert_that(vmfs, instance_of(UnityLun))

    @patch_rest
    def test_create_nfs_share_success(self):
        pool = UnityPool(_id='pool_5', cli=t_rest())
        nas_server = UnityNasServer.get(cli=t_rest(), _id='nas_6')
        job = pool.create_nfs_share(
            nas_server,
            name='513dd8b0-2c22-4da0-888e-494d320303b6',
            size=4294967296)
        assert_that(JobStateEnum.COMPLETED, equal_to(job.state))

    @patch_rest
    def test_create_nfs_share_failed(self):
        def f():
            pool = UnityPool(_id='pool_1', cli=t_rest())
            nas_server = UnityNasServer.get(cli=t_rest(), _id='nas_1')
            pool.create_nfs_share(
                nas_server,
                name='job_share_failed',
                size=1)

        assert_that(f, raises(JobStateError, 'too small'))

    @patch_rest
    def test_create_lun_with_snap_schedule(self):
        pool = UnityPool(_id='pool_1', cli=t_rest())
        schedule = UnitySnapSchedule(_id='snapSch_1', cli=t_rest())
        lun = pool.create_lun(
            lun_name='lun-with-snap-schedule',
            snap_schedule=schedule)
        assert_that(lun.get_id(), equal_to('sv_16455'))



# coding=utf-8
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, contains_string, raises, \
    only_contains

from storops.exception import EnumValueNotFoundError
from storops.unity.enums import HealthEnum

__author__ = 'Cedric Zhuang'


class HealthEnumTest(TestCase):
    def test_parse(self):
        assert_that(HealthEnum.parse(5), equal_to(HealthEnum.OK))

    def test_parse_not_found(self):
        def f():
            HealthEnum.parse(-1)

        assert_that(f, raises(EnumValueNotFoundError, 'HealthEnum'))

    def test_verify(self):
        def f():
            HealthEnum.verify(123)

        assert_that(f, raises(ValueError, 'instance of HealthEnum'))

    def test_verify_none(self):
        HealthEnum.verify(None)
        # do not throw any exception

    def test_to_str(self):
        value = str(HealthEnum.OK)
        assert_that(value, contains_string('"description": "OK"'))
        assert_that(value, contains_string('"value": 5'))
        assert_that(value, contains_string('"name": "OK"'))

    def test_name(self):
        assert_that(HealthEnum.enum_name(), equal_to('HealthEnum'))

    def test_enum_item_name(self):
        assert_that(HealthEnum.MINOR.name, equal_to('MINOR'))

    def test_enum_item_index(self):
        assert_that(HealthEnum.MINOR.index, equal_to(15))

    def test_enum_indices(self):
        assert_that(HealthEnum.indices(),
                    only_contains(0, 5, 7, 10, 15, 20, 25, 30))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

from hamcrest import assert_that, equal_to, contains_string, is_not, none, \
    has_item, close_to

from storops.lib.common import instance_cache
from storops_test.vnx.cli_mock import t_cli, patch_cli, t_vnx
from storops.vnx.enums import VNXSPEnum
from storops.vnx.resource.vnx_domain import VNXDomainNodeList, \
    VNXNetworkAdmin, VNXStorageProcessor, VNXStorageProcessorList

__author__ = 'Cedric Zhuang'


class VNXStorageProcessorTest(TestCase):
    @patch_cli
    def test_sp_list_update_without_poll(self):
        sp_list = self.get_sp_list()
        with sp_list.with_no_poll():
            sp_list.update()
        assert_that(sp_list.total_reads, has_item(1234))
        assert_that(sp_list.timestamp, is_not(none()))

    def get_sp_list(self):
        vnx = t_vnx()
        sp_list = VNXStorageProcessorList(vnx.spa, vnx.spb)
        return sp_list

    @patch_cli
    def test_default_metric_csv_filename(self):
        sp_list = self.get_sp_list()
        filename = sp_list.get_default_metric_csv_filename()
        assert_that(filename, contains_string('.storops'))
        assert_that(filename,
                    contains_string('_VNXStorageProcessor.csv'))

    @patch_cli
    def test_sp_properties(self):
        sp = VNXStorageProcessor(t_cli(), VNXSPEnum.SP_A, '1.1.1.2')
        assert_that(sp.cabinet, equal_to('DPE9'))
        assert_that(sp.signature, equal_to(4022290))
        assert_that(sp.name, equal_to('A'))
        assert_that(sp.enum, equal_to(VNXSPEnum.SP_A))

        sp = VNXStorageProcessor(t_cli(), VNXSPEnum.SP_B, '1.1.1.3')
        assert_that(sp.cabinet, equal_to('DPE9'))
        assert_that(sp.signature, equal_to(4022287))
        assert_that(sp.name, equal_to('B'))
        assert_that(sp.revision, equal_to('05.33.008.3.297'))
        assert_that(sp.serial, equal_to('FCNJT152200015'))
        assert_that(sp.memory_size, equal_to(32768))
        assert_that(sp.enum, equal_to(VNXSPEnum.SP_B))
        assert_that(sp.statistics_logging, equal_to(True))
        assert_that(sp.system_fault_led, equal_to(False))
        assert_that(sp.read_cache_enabled, equal_to(True))
        assert_that(sp.write_cache_enabled, equal_to(True))
        assert_that(sp.max_requests, none())
        assert_that(sp.average_requests, none())
        assert_that(sp.hard_errors, none())
        assert_that(sp.total_reads, equal_to(7978))
        assert_that(sp.total_writes, equal_to(6364257))
        assert_that(sp.prct_busy, equal_to(1.91))
        assert_that(sp.prct_idle, equal_to(98.0))
        assert_that(str(sp.system_timestamp), equal_to('2016-04-26 09:59:16'))
        assert_that(sp.day_of_the_week, equal_to('Tuesday'))
        assert_that(sp.read_requests, equal_to(8308))
        assert_that(sp.write_requests, equal_to(6364593))
        assert_that(sp.blocks_read, equal_to(975038))
        assert_that(sp.blocks_written, equal_to(199037219))
        assert_that(sp.sum_queue_lengths_by_arrivals, equal_to(8059572))
        assert_that(sp.arrivals_to_non_zero_queue, equal_to(1321389))
        assert_that(sp.hw_flush_on, equal_to(False))
        assert_that(sp.idle_flush_on, equal_to(False))
        assert_that(sp.lw_flush_off, equal_to(False))
        assert_that(sp.write_cache_flushes, equal_to(468636))
        assert_that(sp.write_cache_blocks_flushed, equal_to(477297069))
        assert_that(sp.controller_busy_ticks, equal_to(46703))
        assert_that(sp.controller_idle_ticks, equal_to(2394689))

    @property
    @instance_cache
    def sp_a(self):
        return VNXStorageProcessor(t_cli(), VNXSPEnum.SP_A, '10.244.211.30')

    @patch_cli
    def test_sp_read_iops(self):
        assert_that(self.sp_a.read_iops, equal_to(5.5))

    @patch_cli
    def test_sp_write_iops(self):
        assert_that(self.sp_a.write_iops, equal_to(5.6))

    @patch_cli
    def test_sp_total_iops(self):
        assert_that(self.sp_a.total_iops, equal_to(11.1))

    @patch_cli
    def test_sp_read_mbps(self):
        assert_that(self.sp_a.read_mbps, equal_to(5.8))

    @patch_cli
    def test_sp_write_mbps(self):
        assert_that(self.sp_a.write_mbps, equal_to(5.9))

    @patch_cli
    def test_sp_total_mbps(self):
        assert_that(self.sp_a.total_mbps, equal_to(11.7))

    @patch_cli
    def test_sp_read_size_kb(self):
        assert_that(self.sp_a.read_size_kb, close_to(1079.8, 0.1))

    @patch_cli
    def test_sp_write_size_kb(self):
        assert_that(self.sp_a.write_size_kb, close_to(1078.8, 0.1))


class VNXDomainNodeListTest(TestCase):
    @patch_cli
    def setUp(self):
        self.dnl = VNXDomainNodeList(t_cli())

    @patch_cli
    def test_iterable(self):
        assert_that(len(self.dnl), equal_to(2))

    @patch_cli
    def test_get_node(self):
        node = self.dnl.get_node('APM00153906536')
        assert_that(len(node.members), equal_to(2))

    @patch_cli
    def test_get_node_not_found(self):
        assert_that(self.dnl.get_node('abcde'), none())

    @patch_cli
    def test_get_spa_check_ip(self):
        node = self.dnl.get_node('APM00152904560')
        assert_that(node.spa.ip, equal_to('192.168.1.94'))
        assert_that(node.spb.ip, equal_to('192.168.1.95'))
        assert_that(node.control_station.ip, equal_to('192.168.1.93'))

    @patch_cli
    def test_get_cs_ip(self):
        assert_that(VNXDomainNodeList.get_cs_ip('APM00153042305', t_cli()),
                    equal_to('10.244.211.32'))


class VNXDomainMemberListTest(TestCase):
    @patch_cli(output='domain_-list_normal.txt')
    def setUp(self):
        dnl = VNXDomainNodeList(t_cli())
        self.dml = dnl[0].members

    @patch_cli
    def test_iterable(self):
        count = 0
        for _ in self.dml:
            count += 1
        assert_that(count, equal_to(3))

    @patch_cli
    def test_properties(self):
        str_value = str(self.dml)
        assert_that(str_value, contains_string('VNXDomainMember'))
        assert_that(str_value, contains_string('VNXDomainMemberList'))
        assert_that(str_value, is_not(contains_string('::')))

    @patch_cli
    def test_sp(self):
        spa = self.dml.spa
        assert_that(spa.ip, equal_to('10.244.211.30'))
        assert_that(spa.is_master, equal_to(True))
        spb = self.dml.spb
        assert_that(spb.ip, equal_to('10.244.211.31'))
        assert_that(spb.is_master, equal_to(False))
        cs = self.dml.control_station
        assert_that(cs.ip, equal_to('10.244.211.32'))


class VNXNetworkAdminTest(TestCase):
    @patch_cli
    def test_properties(self):
        sp = VNXNetworkAdmin(VNXSPEnum.SP_A, t_cli())
        with sp.with_no_poll():
            assert_that(sp.name, equal_to('vnx2_1_52'))
            assert_that(sp.sp, equal_to(VNXSPEnum.SP_A))
            assert_that(sp.link_status, equal_to('Link-Up'))
            assert_that(sp.subnet_mask, equal_to('255.255.255.0'))
            assert_that(sp.ip, equal_to('192.168.1.52'))
            assert_that(sp.ip_mode, equal_to('Manual'))
            assert_that(sp.gateway, equal_to('192.168.1.1'))
            assert_that(sp.virtual_port_id, equal_to(0))
            assert_that(sp.ipv6_enabled, equal_to(False))
            assert_that(sp.port_id, equal_to(0))
            assert_that(sp.vlan_id, equal_to(None))

    @patch_cli
    def test_get_spa_ip(self):
        assert_that(VNXNetworkAdmin.get_spa_ip(t_cli()),
                    equal_to('192.168.1.52'))

    @patch_cli
    def test_get_spb_ip(self):
        assert_that(VNXNetworkAdmin.get_spb_ip(t_cli()),
                    equal_to('192.168.1.53'))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import re
from collections import Counter

from storops.lib.common import check_text, instance_cache
from storops.vnx.resource import VNXCliResource, VNXCliResourceList

__author__ = 'Cedric Zhuang'


class VNXDisk(VNXCliResource):
    _index_pattern = re.compile(r'(\w+)_(\w+)_(\w+)')

    def __init__(self, index=None, cli=None):
        super(VNXDisk, self).__init__()
        self._cli = cli
        self._index = index
        if index is not None:
            self.parse_index(self.index)

    @property
    def bus(self):
        return self.parse_index(self._index)[0]

    @property
    def enclosure(self):
        return self.parse_index(self._index)[1]

    @property
    def disk(self):
        return self.parse_index(self._index)[2]

    @property
    def index(self):
        if self._index is None:
            if self.disk_index is not None:
                self._index = '{}_{}_{}'.format(*self.disk_index)
            else:
                raise ValueError('disk index is not initialized.')
        return self._index

    @property
    def index_string(self):
        bus, enc, disk = self.parse_index(self.index)
        return 'bus {} enclosure {} disk {}'.format(bus, enc, disk)

    def _get_raw_resource(self):
        return self._cli.get_disk(poll=self.poll,
                                  *self.parse_index(self._index))

    @classmethod
    def parse_index(cls, index):
        match = re.search(cls._index_pattern, index)
        if match is None:
            raise ValueError('invalid disk index.  disk index must '
                             'be something like '
                             '"1_2_A0", in which "1" is the bus id, "2"'
                             'is the enclosure id and "A0" is the disk id.')
        return match.groups()

    @classmethod
    def get(cls, cli, index=None):
        if index is None:
            ret = VNXDiskList(cli)
        else:
            index = check_text(index).upper()
            ret = VNXDisk(index, cli)
        return ret

    def delete(self):
        return self._cli.delete_disk(self._index, poll=self.poll)

    def install(self):
        return self._cli.install_disk(self._index, poll=self.poll)


class VNXDiskList(VNXCliResourceList):
    def __init__(self, cli=None, disk_indices=None, drive_type=None,
                 capacity=None):
        super(VNXDiskList, self).__init__(cli)
        self._disk_indices = None
        self._drive_type = None
        self._capacity = None

        self._set_filter(disk_indices, drive_type, capacity)

    @staticmethod
    def _normalize_indices(indices):
        return [index.lower() for index in indices if index is not None]

    def _set_filter(self, disk_indices=None, drive_type=None, capacity=None):
        if disk_indices is not None:
            self._disk_indices = self._normalize_indices(disk_indices)
        else:
            self._disk_indices = None
        self._drive_type = drive_type
        self._capacity = capacity

    def set_indices(self, disk_indices):
        self.set_filter(disk_indices=disk_indices)

    def set_drive_type(self, drive_type):
        self.set_filter(drive_type=drive_type)

    def set_capacity(self, capacity):
        self.set_filter(capacity=capacity)

    def _filter(self, disk):
        ret = True
        if self._disk_indices:
            index = disk.index
            ret &= index and index.lower() in self._disk_indices
        if self._drive_type:
            if disk.drive_type is None:
                ret = False
            else:
                ret &= disk.drive_type.lower() == self._drive_type.lower()
        if self._capacity:
            if disk.capacity is None:
                ret = False
            else:
                ret &= disk.capacity == self._capacity
        return ret

    def same_disks(self, count=2):
        """ filter self to the required number of disks with same size and type

        Select the disks with the same type and same size.  If not
        enough disks available, set self to empty.

        :param count: number of disks to retrieve
        :return: disk list
        """
        ret = self
        if len(self) > 0:
            type_counter = Counter(self.drive_type)
            drive_type, counts = type_counter.most_common()[0]
            self.set_drive_type(drive_type)

        if len(self) > 0:
            size_counter = Counter(self.capacity)
            size, counts = size_counter.most_common()[0]
            self.set_capacity(size)

        if len(self) >= count:
            indices = self.index[:count]
            self.set_indices(indices)
        else:
            self.set_indices('N/A')
        return ret

    @classmethod
    def get_resource_class(cls):
        return VNXDisk

    def _get_raw_resource(self):
        return self._cli.get_disk(poll=self.poll)

    @property
    @instance_cache
    def _disk_index_map(self):
        return {disk.index: disk for disk in self}

    def get(self, index):
        if isinstance(index, VNXDisk):
            index = index.index

        return self._disk_index_map.get(index)



# Copyright (c) 2018 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import unicode_literals

import logging

from storops.unity.resource import UnityResource, UnityResourceList, \
    UnityAttributeResource

__author__ = 'Ryan Liang'

log = logging.getLogger(__name__)


class UnityLunMemberReplication(UnityAttributeResource):
    @staticmethod
    def to_embedded(src_lun_id=None, dst_lun_id=None, src_status=None,
                    network_status=None, dst_status=None):
        return {'srcLunId': src_lun_id, 'dstLunId': dst_lun_id,
                'srcStatus': src_status, 'dstStatus': dst_status,
                'networkStatus': network_status}


class UnityLunMemberReplicationList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityLunMemberReplication


class UnitySnapReplicationPolicy(UnityAttributeResource):
    @staticmethod
    def to_embedded(is_replicating_snaps=None,
                    is_retention_same_as_source=None,
                    is_auto_delete=None, retention_duration=None):
        return {'isReplicatingSnaps': is_replicating_snaps,
                'isRetentionSameAsSource': is_retention_same_as_source,
                'isAutoDelete': is_auto_delete,
                'retentionDuration': retention_duration}


class UnityResourceConfig(UnityAttributeResource):
    @staticmethod
    def to_embedded(pool_id=None, is_thin_enabled=None,
                    is_deduplication_enabled=None, is_compression_enabled=None,
                    is_backup_only=None, size=None, tiering_policy=None,
                    request_id=None, src_id=None, name=None, default_sp=None,
                    replication_resource_type=None):
        """
        Constructs an embeded object of `UnityResourceConfig`.

        :param pool_id: storage pool of the resource.
        :param is_thin_enabled: is thin type or not.
        :param is_deduplication_enabled: is deduplication enabled or not.
        :param is_compression_enabled: is in-line compression (ILC) enabled or
            not.
        :param is_backup_only: is backup only or not.
        :param size: size of the resource.
        :param tiering_policy: `TieringPolicyEnum` value. Tiering policy
            for the resource.
        :param request_id: unique request ID for the configuration.
        :param src_id: storage resource if it already exists.
        :param name: name of the storage resource.
        :param default_sp: `NodeEnum` value. Default storage processor for
            the resource.
        :param replication_resource_type: `ReplicationEndpointResourceTypeEnum`
            value. Replication resource type.
        :return:
        """
        return {'poolId': pool_id, 'isThinEnabled': is_thin_enabled,
                'isDeduplicationEnabled': is_deduplication_enabled,
                'isCompressionEnabled': is_compression_enabled,
                'isBackupOnly': is_backup_only, 'size': size,
                'tieringPolicy': tiering_policy, 'requestId': request_id,
                'srcId': src_id, 'name': name, 'defaultSP': default_sp,
                'replicationResourceType': replication_resource_type}


class UnityResourceInfo(UnityAttributeResource):
    @staticmethod
    def to_embedded(res_id=None, name=None, system=None):
        """
        Constructs an embeded object of `UnityResourceInfo`.

        :param res_id: identifier of the resource.
        :param name: name of the resource.
        :param system: system on which the resource exists.
        :return:
        """
        return {'resId': res_id, 'name': name, 'system': system}


class UnityReplicationSession(UnityResource):

    @classmethod
    def get_nested_properties(cls):
        return (
            'remoteSystem.name',
        )

    @classmethod
    def create(cls, cli, src_resource_id, dst_resource_id,
               max_time_out_of_sync, name=None, members=None,
               auto_initiate=None, hourly_snap_replication_policy=None,
               daily_snap_replication_policy=None,
               replicate_existing_snaps=None, remote_system=None,
               src_spa_interface=None, src_spb_interface=None,
               dst_spa_interface=None, dst_spb_interface=None):
        """
        Creates a replication session.

        :param cli: the rest cli.
        :param src_resource_id: id of the replication source, could be
            lun/fs/cg.
        :param dst_resource_id: id of the replication destination.
        :param max_time_out_of_sync: maximum time to wait before syncing the
            source and destination. Value `-1` means the automatic sync is not
            performed. `0` means it is a sync replication.
        :param name: name of the replication.
        :param members: list of `UnityLunMemberReplication` object. If
            `src_resource` is cg, `lunMemberReplication` list need to pass in
            to this parameter as member lun pairing between source and
            destination cg.
        :param auto_initiate: indicates whether to perform the first
            replication sync automatically.
            True - perform the first replication sync automatically.
            False - perform the first replication sync manually.
        :param hourly_snap_replication_policy: `UnitySnapReplicationPolicy`
            object. The policy for replicating hourly scheduled snaps of the
            source resource.
        :param daily_snap_replication_policy: `UnitySnapReplicationPolicy`
            object. The policy for replicating daily scheduled snaps of the
            source resource.
        :param replicate_existing_snaps: indicates whether or not to replicate
            snapshots already existing on the resource.
        :param remote_system: `UnityRemoteSystem` object. The remote system of
            remote replication.
        :param src_spa_interface: `UnityRemoteInterface` object. The
            replication interface for source SPA.
        :param src_spb_interface: `UnityRemoteInterface` object. The
            replication interface for source SPB.
        :param dst_spa_interface: `UnityRemoteInterface` object. The
            replication interface for destination SPA.
        :param dst_spb_interface: `UnityRemoteInterface` object. The
            replication interface for destination SPB.
        :return: the newly created replication session.
        """

        req_body = cli.make_body(
            srcResourceId=src_resource_id, dstResourceId=dst_resource_id,
            maxTimeOutOfSync=max_time_out_of_sync, members=members,
            autoInitiate=auto_initiate, name=name,
            hourlySnapReplicationPolicy=hourly_snap_replication_policy,
            dailySnapReplicationPolicy=daily_snap_replication_policy,
            replicateExistingSnaps=replicate_existing_snaps,
            remoteSystem=remote_system,
            srcSPAInterface=src_spa_interface,
            srcSPBInterface=src_spb_interface,
            dstSPAInterface=dst_spa_interface,
            dstSPBInterface=dst_spb_interface)

        resp = cli.post(cls().resource_class, **req_body)
        resp.raise_if_err()
        return cls.get(cli, resp.resource_id)

    @classmethod
    def create_with_dst_resource_provisioning(
            cls, cli, src_resource_id, dst_resource_config,
            max_time_out_of_sync, name=None, remote_system=None,
            src_spa_interface=None, src_spb_interface=None,
            dst_spa_interface=None, dst_spb_interface=None,
            dst_resource_element_configs=None, auto_initiate=None,
            hourly_snap_replication_policy=None,
            daily_snap_replication_policy=None, replicate_existing_snaps=None,
            no_async_snap_replication=None,
    ):
        """
        Create a replication session along with destination resource
        provisioning.

        :param cli: the rest cli.
        :param src_resource_id: id of the replication source, could be
            lun/fs/cg.
        :param dst_resource_config: `UnityResourceConfig` object. The user
            chosen config for destination resource provisioning. `pool_id` and
            `size` are required for creation.
        :param max_time_out_of_sync: maximum time to wait before syncing the
            source and destination. Value `-1` means the automatic sync is not
            performed. `0` means it is a sync replication.
        :param name: name of the replication.
        :param remote_system: `UnityRemoteSystem` object. The remote system to
            which the replication is being configured. When not specified, it
            defaults to local system.
        :param src_spa_interface: `UnityRemoteInterface` object. The
            replication interface for source SPA.
        :param src_spb_interface: `UnityRemoteInterface` object. The
            replication interface for source SPB.
        :param dst_spa_interface: `UnityRemoteInterface` object. The
            replication interface for destination SPA.
        :param dst_spb_interface: `UnityRemoteInterface` object. The
            replication interface for destination SPB.
        :param dst_resource_element_configs: List of `UnityResourceConfig`
            objects. The user chose config for each of the member element of
            the destination resource.
        :param auto_initiate: indicates whether to perform the first
            replication sync automatically.
            True - perform the first replication sync automatically.
            False - perform the first replication sync manually.
        :param hourly_snap_replication_policy: `UnitySnapReplicationPolicy`
            object. The policy for replicating hourly scheduled snaps of the
            source resource.
        :param daily_snap_replication_policy: `UnitySnapReplicationPolicy`
            object. The policy for replicating daily scheduled snaps of the
            source resource.
        :param replicate_existing_snaps: indicates whether or not to replicate
            snapshots already existing on the resource.
        :param no_async_snap_replication: whether or not snap replication is
            enabled in asynchronous replication session. When enabled, snap
            replication is controlled by snap replication policy setting or
            user action.
        :return: the newly created replication session.
        """

        req_body = cli.make_body(
            srcResourceId=src_resource_id,
            dstResourceConfig=dst_resource_config,
            maxTimeOutOfSync=max_time_out_of_sync,
            name=name, remoteSystem=remote_system,
            srcSPAInterface=src_spa_interface,
            srcSPBInterface=src_spb_interface,
            dstSPAInterface=dst_spa_interface,
            dstSPBInterface=dst_spb_interface,
            dstResourceElementConfigs=dst_resource_element_configs,
            autoInitiate=auto_initiate,
            hourlySnapReplicationPolicy=hourly_snap_replication_policy,
            dailySnapReplicationPolicy=daily_snap_replication_policy,
            replicateExistingSnaps=replicate_existing_snaps,
            noAsyncSnapReplication=no_async_snap_replication,
        )

        resp = cli.type_action(
            cls().resource_class,
            'createReplicationSessionWDestResProvisioning',
            **req_body)
        resp.raise_if_err()
        # response is like:
        # "content": {
        #     "id": {
        #         "id": "42949676351_FNM00150600267_xxxx"
        #     }
        session_resp = resp.first_content['id']
        return cls.get(cli, _id=session_resp['id'])

    def modify(self, max_time_out_of_sync=None, name=None,
               hourly_snap_replication_policy=None,
               daily_snap_replication_policy=None,
               src_spa_interface=None, src_spb_interface=None,
               dst_spa_interface=None, dst_spb_interface=None):
        """
        Modifies properties of a replication session.

        :param max_time_out_of_sync: same as the one in `create` method.
        :param name: same as the one in `create` method.
        :param hourly_snap_replication_policy: same as the one in `create`
            method.
        :param daily_snap_replication_policy: same as the one in `create`
            method.
        :param src_spa_interface: same as the one in `create` method.
        :param src_spb_interface: same as the one in `create` method.
        :param dst_spa_interface: same as the one in `create` method.
        :param dst_spb_interface: same as the one in `create` method.
        """
        req_body = self._cli.make_body(
            maxTimeOutOfSync=max_time_out_of_sync, name=name,
            hourlySnapReplicationPolicy=hourly_snap_replication_policy,
            dailySnapReplicationPolicy=daily_snap_replication_policy,
            srcSPAInterface=src_spa_interface,
            srcSPBInterface=src_spb_interface,
            dstSPAInterface=dst_spa_interface,
            dstSPBInterface=dst_spb_interface)

        resp = self.action('modify', **req_body)
        resp.raise_if_err()
        return resp

    def resume(self, force_full_copy=None,
               src_spa_interface=None, src_spb_interface=None,
               dst_spa_interface=None, dst_spb_interface=None):
        """
        Resumes a replication session.

        This can be applied on replication session when it's operational status
        is reported as Failed over, or Paused.

        :param force_full_copy: needed when replication session goes out of
            sync due to a fault.
            True - replicate all data.
            False - replicate changed data only.
        :param src_spa_interface: same as the one in `create` method.
        :param src_spb_interface: same as the one in `create` method.
        :param dst_spa_interface: same as the one in `create` method.
        :param dst_spb_interface: same as the one in `create` method.
        """
        req_body = self._cli.make_body(forceFullCopy=force_full_copy,
                                       srcSPAInterface=src_spa_interface,
                                       srcSPBInterface=src_spb_interface,
                                       dstSPAInterface=dst_spa_interface,
                                       dstSPBInterface=dst_spb_interface)

        resp = self.action('resume', **req_body)
        resp.raise_if_err()
        return resp

    def pause(self):
        """
        Pauses a replication session.

        This can be applied on replication session when in `OK` state.
        """
        resp = self.action('pause')
        resp.raise_if_err()
        return resp

    def sync(self):
        """
        Syncs a replication session.

        This can be applied to initiate a sync on demand independent of type of
        replication session - auto or manual sync.
        """
        resp = self.action('sync')
        resp.raise_if_err()
        return resp

    def failover(self, sync=None, force=None):
        """
        Fails over a replication session.

        :param sync: True - sync the source and destination resources before
            failing over the asynchronous replication session or keep them in
            sync after failing over the synchronous replication session.
            False - don't sync.
        :param force: True - skip pre-checks on file system(s) replication
            sessions of a NAS server when a replication failover is issued from
            the source NAS server.
            False - don't skip pre-checks.
        """
        req_body = self._cli.make_body(sync=sync, force=force)
        resp = self.action('failover', **req_body)
        resp.raise_if_err()
        return resp

    def failback(self, force_full_copy=None):
        """
        Fails back a replication session.

        This can be applied on a replication session that is failed over. Fail
        back will synchronize the changes done to original destination back to
        original source site and will restore the original direction of
        session.

        :param force_full_copy: indicates whether to sync back all data from
            the destination SP to the source SP during the failback session.
            True - Sync back all data.
            False - Sync back changed data only.
        """
        req_body = self._cli.make_body(forceFullCopy=force_full_copy)
        resp = self.action('failback', **req_body)
        resp.raise_if_err()
        return resp


class UnityReplicationSessionList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityReplicationSession

    @staticmethod
    def filter_by_source_destination(src_system_name, src_resource_id,
                                     dst_system_name, dst_resource_id):
        UnityReplicationSessionList.get()
